'use strict';


var Licenses = {


    SOFTWARE: [
        // Framework
        {
            name: 'NW.js',
            file: 'mediola/licenses/files/nwjs.txt',
            type: 'MIT'
        },
        // Node modules
        {
            name: 'aws-sdk',
            file: 'node_modules/aws-sdk/LICENSE.txt',
            type: 'Apache-2.0'
        },
        {
            name: 'base64-js',
            file: 'node_modules/base64-js/LICENSE',
            type: 'MIT'
        },
        {
            name: 'buffer',
            file: 'node_modules/buffer/LICENSE',
            type: 'MIT'
        },
        {
            name: 'crypto-browserify',
            file: 'node_modules/crypto-browserify/LICENSE',
            type: 'MIT'
        },
        {
            name: 'ieee754',
            file: 'node_modules/ieee754/LICENSE',
            type: 'BSD-3-Clause'
        },
        {
            name: 'isarray',
            file: 'mediola/licenses/files/isarray.txt',
            type: 'MIT'
        },
        {
            name: 'jmespath',
            file: 'node_modules/jmespath/LICENSE',
            type: 'Apache-2'
        },
        {
            name: 'lodash',
            file: 'node_modules/lodash/LICENSE.txt',
            type: 'MIT'
        },
        {
            name: 'punycode',
            file: 'node_modules/punycode/LICENSE-MIT.txt',
            type: 'MIT'
        },
        {
            name: 'querystring',
            file: 'node_modules/querystring/License.md',
            type: 'MIT'
        },
        {
            name: 'sax',
            file: 'node_modules/sax/LICENSE',
            type: 'ISC'
        },
        {
            name: 'url',
            file: 'node_modules/url/LICENSE',
            type: 'MIT'
        },
        {
            name: 'uuid',
            file: 'node_modules/uuid/LICENSE.md',
            type: 'MIT'
        },
        {
            name: 'xml2js',
            file: 'node_modules/xml2js/LICENSE',
            type: 'MIT'
        },
        {
            name: 'xmlbuilder',
            file: 'node_modules/xmlbuilder/LICENSE',
            type: 'MIT'
        },
    ],


    /**
     * Build a list item.
     * @param  {Object}      data
     * @return {HTMLElement}
     */
    _buildItem: function(data) {
        var item = document.createElement('li');
        var name = document.createElement('span');
        var type = document.createElement('span');

        name.className = 'name';
        name.textContent = data.name;

        type.className = 'type';
        type.textContent = data.type;

        item.appendChild(name);
        item.appendChild(type);

        item.setAttribute('data-file', data.file);
        item.addEventListener('click', this._loadLicense.bind(this));

        return item;
    },


    /**
     * Load the license.
     * @param {Event} ev
     */
    _loadLicense: function(ev) {
        var item = ev.currentTarget || ev.target;
        var filepath = item.getAttribute('data-file');
        var textArea = document.getElementById('license');

        if (!filepath) {
            console.error('[Licenses._loadLicense] No filepath.');
            return;
        }

        var active = document.querySelectorAll('#software li.active');

        for (var i = 0; i < active.length; i++) {
            active[i].className = '';
        }

        item.className = 'active';

        var fs = require('fs');
        var options = {
            encoding: 'utf-8',
            flag: 'r'
        };

        fs.readFile(filepath, options, function(err, res) {
            var content = '';

            if (err) {
                console.error('[Licenses._loadLicense] ' + err.message);
                content = 'Lizenzdatei konnte nicht geladen werden.';
            }
            else {
                content = String(res);
            }

            textArea.textContent = content;
        });
    },


    /**
     * Initialize the UI.
     */
    init: function() {
        var list = document.getElementById('software');
        list.innerHTML = '';

        this.SOFTWARE.sort(function(a, b) {
            return a.name.localeCompare(b.name);
        });

        for (var i = 0; i < this.SOFTWARE.length; i++) {
            var software = this.SOFTWARE[i];
            var item = this._buildItem(software);
            list.appendChild(item);
        }
    }


};
