/**
 * Page class
 * @constructor
 */
MCAPage = function(config, path){
    var self = this;
    this.activeElement = null;
    this._config = config;
    this._path = path;

    this.ui = {
        'borderColor' : '#ccc',
        'btnColor' : '#999',
        'btnDownColor' : '#444',
        'config' : {
            'tilesPerRow' : 5,
            'rows' : 2,
            'dangerZone' : 80
        },
        'background' : '#000',
        'overlay' : {
            'background' : '#222',
            'opacity' : 0.9
        }
    };
    this.inputPopVisible = false;
    this.sizes = {
       //width : 2048,
       //height : 1536,
       //scaleFactor : 1
       width : 1024,
       space : 115,
       height : 768,
       scaleFactor : 0.5
    };

    //this.sizes.heightTopBar = this.sizes.height * 0.15;
    this.sizes.heightBottomBar = this.sizes.height * 0.2;
    this.sizes.heightTilesContainer = this.sizes.height * 0.65;
    this.sizes.pageHeight = this.sizes.heightBottomBar + this.sizes.heightTilesContainer;

    this.$container = $('<div></div>');
    this.$container.css({
        'width' : this.sizes.width + 'px',
        'height' : this.sizes.pageHeight + 'px'
    });

    /* this._$body = $('body');
    this._$body.css({
        'width' : this.sizes.width + 'px',
        'height' : this.sizes.pageHeight + 'px'
    }); */


    //this.$topBar = $('<div style="background: '+this.ui.background+'; position:absolute; top:0px;height:'+this.sizes.heightTopBar+'px;width:'+this.sizes.width+'px"></div>');
    this.$tilesContainer = $('<div class="update" style="background: '+this.ui.background+'; position:absolute; top:0px; height:'+this.sizes.heightTilesContainer+'px;width:'+this.sizes.width+'px"></div>');
    this.$bottomButtonContainer = $('<div class="update" style="background: '+this.ui.background+'; position:absolute; top: '+ (this.sizes.heightTilesContainer+this.sizes.space)+'px;height:'+this.sizes.heightBottomBar+'px;width:'+this.sizes.width+'px; text-align:center;"></div>');
    this.$spinner = $('<div style="z-index:300;"><img src="'+this._path+'spinner.gif" /></div>');
    //this.$spinnerOverlay = $('<div style="background:none; position: absolute; left: 0px; top: 0px; z-index: 299; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this._$body);
    this.$spinnerOverlay = $('<div style="background:none; position: absolute; left: 0px; top: 0px; z-index: 299; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this.$container);
    this.$popUpPartyMode = new PartyModePopUp(this);
    this.$popUpZone = new ZonePopUp(this);
    //this.$popUpInput = new AmpInputPopup(this);

    //this._$body.append(this.$tilesContainer)
    this.$container.append(this.$tilesContainer)
        .append(this.$bottomButtonContainer);
        //.append(this.$popUpPartyMode);

    this.$bottomButtonContainer.append(this.$spinner);

    /*this.$tilesContainer.appendTo(this.$container);
    this.$bottomButtonContainer.appendTo(this.$container);
    this.$spinner.appendTo(this.$bottomButtonContainer);*/



    //this._$body.bind('mousedown touchstart', function(e) {
    this.$container.bind('mousedown touchstart', function(e) {
        e.preventDefault();

        //check if all Vol slider is visible
        if(self.$allVolSliderDiv.height() != 0) {
            self.toggleAllVolSlider(e);
        }

        //Update UI
        if(e.target.className == "update") {
            //update UI if obj with update class is pressed */
            self.showSpinner();

            var callback = null;
            if(amp2) {
                callback = amp2.updateUI();
            }
            amp1.updateUI(callback);
        }

    });

    this._init();
};

/**
 * page init function
 * @private
 */
MCAPage.prototype._init = function(){

    var self = this;

    //add start page push buttons
    var containerHeight = this.$bottomButtonContainer.height(),
        height = 184 * this.sizes.scaleFactor, //real image height
        width = 184 * this.sizes.scaleFactor, // real image width
        marginTop = -55; //containerHeight - height ;

    /*this.$bottomButtonContainer.css({
        height : 92 + 'px'
    });*/

    //All on
    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/alle-an.png',
        'height' : height,
        'width' : width,
        'margin' : marginTop + 'px 0 0 0',
        'container' : this.$bottomButtonContainer,
        callback : function(){
            self.showSpinner();
            amp1.powerOnZones();
            if(amp2) {
                amp2.powerOnZones();
            }
        }
    });

    //All off
    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/alle-aus.png',
        'height' : height,
        'width' : width,
        'margin' : marginTop + 'px '+width*3 +'px 0 0',
        'container' : this.$bottomButtonContainer,
        callback : function(){
            self.showSpinner();
            amp1.powerOffZones();
            if(amp2) {
               amp2.powerOffZones();
            }
        }
    });

    this.allVolHeight = 600 * this.sizes.scaleFactor;
    var allVolLeft = 1300 * this.sizes.scaleFactor;

    //this.$allVolSliderDiv = $('<div></div>').appendTo(this.$bottomButtonContainer);
    this.$allVolSliderDiv = $('<div></div>').appendTo(this.$container);
    this.$allVolSliderDiv.css({
        'height' : '0px',
        'width' : width + 'px',
        'position' : 'absolute',
        'bottom' : (height + 2) + 'px',
        'display' : 'none',
        'left' : allVolLeft + 'px',
        'background' : 'url("'+this._path +'bg/bg_allVol_slider.png") no-repeat #000',
        'background-size': width + 'px '+ this.allVolHeight +'px',
        'z-index' : '80'
    });

    var spinner = {
        height : (220 * this.sizes.scaleFactor) * 0.6,
        width : (220 * this.sizes.scaleFactor) * 0.6,
        bottom : 0
    };

    this.$spinner.css({
        'height' : spinner.height + 'px',
        'width' : spinner.width + 'px',
        'position' : 'absolute',
        'top' : '-45px',
        'left' : (this.sizes.width - spinner.width)/2 + 'px',
        'display' : 'none'
    });

    this.$spinner.find('img').css({
        height : spinner.height + 'px',
        width : spinner.width + 'px'
    });

    var allVolSlider = {
        left : (width - 27.5)/2 + 3,
        top : -3
    };

    //Add Balance Slider
    this._allVolSlider  = new MSliderAmp(this._path + 'slider/', this);
    this._allVolSlider.hasBackground(false);
    this._allVolSlider.indicator = true;
    this._allVolSlider.setVertical(true);
    this._allVolSlider.setRange(-25,25);
    this._allVolSlider.step = 5;
    this._allVolSlider.addToPage(this.$allVolSliderDiv, allVolSlider.left, allVolSlider.top, this.allVolHeight);
    this._allVolSlider.setState(0);
    this._allVolSlider.endCallback = function(delta){
        self.showSpinner();

        var callback = function(){
            self._allVolSlider.setState(0);
            self.hideSpinner();
        };

        if(amp2) {
            callback = function(){
                amp2.allVolDown(delta, function(){
                    amp2.updateUI();
                    self._allVolSlider.setState(0);
                    self.hideSpinner();
                });
            };
        }

        setTimeout(function(){
            if(delta < 0) {

                amp1.allVolDown(delta, function(){
                    amp1.updateUI(callback);
                });
            } else if(delta > 0) {
                amp1.allVolUp(delta, function(){
                    amp1.updateUI(callback);
                });
            } else {
                self.hideSpinner();
            }
        }, 500);
    };
    this._allVolSlider.hide();

    //All vol up
    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/alle-lauter.png',
        'height' : height,
        'width' : width,
        'margin' : marginTop + 'px 0 0 0',
        'container' : this.$bottomButtonContainer,
        'zindex' : '90',
        callback : function(e){
            self.toggleAllVolSlider(e);
        }
    });

    //party mode
    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/modus.png',
        'height' : height,
        'width' : width,
        'margin' : marginTop + 'px 0 0 0',
        'container' : this.$bottomButtonContainer,
        callback : function(){
            var amps = [amp1];
            if(amp2) {
                amps.push(amp2);
            }
            self.$popUpPartyMode.show(amps);
        }
    });



    //excute queue
    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/start.png',
        'height' : height,
        'width' : width,
        'top' : marginTop,
        'right' : '0',
        'container' : this.$bottomButtonContainer,
        callback : function(){
            self.showSpinner();

            MCAAjaxManager.setRunCallback(function(){
                var callback = null;
                if(amp2) {
                    callback = function(){
                        amp2.updateUI();
                    }
                }
                amp1.updateUI(callback);
            });

            MCAAjaxManager.run();
        }
    });
};

/**
 * Toggles the all volume slider
 * @returns {*|$}
 */
MCAPage.prototype.toggleAllVolSlider = function(e){
    var self = this;

    e.preventDefault();

    if(this.$allVolSliderDiv.height() != 0) {

        self._allVolSlider.hide();
        this.$allVolSliderDiv.animate({
            height : '0px'
        }, 500, false, function (){

            self.$allVolSliderDiv.css({
                display: 'none'
            });

        });

    } else {

        this.$allVolSliderDiv.css({
            display: 'block'
        });

        this.$allVolSliderDiv.animate({
            height: this.allVolHeight + 'px'
        }, 500, false, function(){
            self._allVolSlider.show()
        });
    }
};

/**
 * Shows the loading spinner
 */
MCAPage.prototype.showSpinner = function(){
    this.$spinner.css({
       'display' : 'block'
    });

    this.$spinnerOverlay.css({
        'display' : 'block'
    });
};

/**
 * Hides the loading spinner
 */
MCAPage.prototype.hideSpinner = function(){
    var self = this;
    //if (this.$spinner.is(':visible')) {

        //setTimeout(function() {
            self.$spinner.css({
                'display': 'none'
            });

            self.$spinnerOverlay.css({
                'display': 'none'
            });
        //}, 500);
    //}
};

/**
 * Displays PI connection error popup
 */
MCAPage.prototype.showPIError = function(){

    var self = this;

    var imageHeight = 115 * this.sizes.scaleFactor,
        imageWidth = 718 * this.sizes.scaleFactor,
        windowWidth = imageWidth+200,
        windowHeight = this.sizes.pageHeight/ 2;

    //this._$body.css({ 'background' : '#000'});
    this.$container.css({ 'background' : '#000'});
    //this.$PIErrorOverlay = $('<div style="position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this._$body);
    this.$PIErrorOverlay = $('<div style="position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this.$container);
    //this.$PIErrorWindow = $('<div style="z-index: 101; position:absolute; height:'+windowHeight+'px; width:'+windowWidth +'px;text-align:center;"><p style="color:#fff; margin-bottom:40px;">Verbindung zum Server nicht möglich! Bitte Server überprüfen!</p><img style="position: relative; bottom: 0; width:'+imageWidth+'px; height:'+imageHeight+'px" src="images/error/amp_pop-up_disconnected_ipad_server.png" /></div>').appendTo(this._$body);
    this.$PIErrorWindow = $('<div style="z-index: 101; position:absolute; height:'+windowHeight+'px; width:'+windowWidth +'px;text-align:center;"><p style="color:#fff; margin-bottom:40px;">Verbindung zum Server nicht möglich! Bitte Server überprüfen!</p><img style="position: relative; bottom: 0; width:'+imageWidth+'px; height:'+imageHeight+'px" src='+this._path + '"error/amp_pop-up_disconnected_ipad_server.png" /></div>').appendTo(this.$container);

    this.$PIErrorOverlay.css({
        'background' : this.ui.overlay.background,
        'opacity' : this.ui.overlay.opacity
    });

    this.$PIErrorWindow.css({
       'top' : (this.sizes.pageHeight/2)/2 + 'px',
        'left' : ((this.sizes.width - windowWidth)/2) + 'px',
        'border-style': 'solid',
        'border-width ': '1px',
        'border-color' : 'rgb(0, 0, 0)',
        'border-radius': '-6px',
        'background-color' : '#000',
        'box-shadow' : '0px 2px 0px 0px rgba(35, 35, 35, 0.54), inset 0px 3px 0px 0px rgba(255, 255, 255, 0.15)'
    });

    var height = 184 * this.sizes.scaleFactor, //real image height
        width = 184 * this.sizes.scaleFactor, // real image width
        left = (windowWidth - width) / 2,
        top = windowHeight - height;

    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/reload.png',
        'height' : height,
        'width' : width,
        'left' : left,
        'top' : top,
        'container' : this.$PIErrorWindow,
        callback : function(){
            //console.log('partymode popup');
            self.removePIError();
            MCAPi.getHeartBeat(function(){
                MCAPi.initAmps();
            });
        }
    });
};


/**
 * Removes PI connection error popup
 */
MCAPage.prototype.removePIError = function(){
    this.$PIErrorOverlay.remove();
    this.$PIErrorWindow.remove();
};

/**
 * Displays Amp Connection Error window
 */
MCAPage.prototype.showAmpConnectionError = function(amp){

    //hide zone popup if visible
    if(typeof this.$popUpZone != 'undefined') {
        //if(this.$popUpZone.$popUp.is(':visible')) {
            this.$popUpZone.hide();
        //}
    }

    //hide partyzone popup if visible
    if(typeof this.$popUpPartyMode != 'undefined') {
        //if(this.$popUpPartyMode.$popUp.is(':visible')) {
            this.$popUpPartyMode.hide();
        //}
    }

    var self = this;

    var imageHeight = 115 * this.sizes.scaleFactor,
        imageWidth = 920 * this.sizes.scaleFactor,
        windowWidth = imageWidth + 200,
        windowHeight = this.sizes.pageHeight/2;

    //this.$ampErrorOverlay = $('<div style="position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this._$body);
    this.$ampErrorOverlay = $('<div style="position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.sizes.width+'px; height:'+this.sizes.pageHeight+'px; text-align: center; no-repeat center"></div>').appendTo(this.$container);
    var ampErrorText = 'Verbindung zu Verstärker '+amp+' unterbrochen! Bitte Verstärker und Kabel prüfen!';
    //this.$ampErrorWindow = $('<div style="z-index: 101; position:absolute; height:'+windowHeight+'px; width:'+windowWidth+'px;text-align:center;"><p style="color:#fff; margin-bottom:40px;">'+ampErrorText+'</p><img style="position: relative; bottom: 0; width:'+imageWidth+'px; height:'+imageHeight+'px" src="images/error/amp_pop-up_disconnected_server_reciever.png" /></div>').appendTo(this._$body);
    this.$ampErrorWindow = $('<div style="z-index: 101; position:absolute; height:'+windowHeight+'px; width:'+windowWidth+'px;text-align:center;"><p style="color:#fff; margin-bottom:40px;">'+ampErrorText+'</p><img style="position: relative; bottom: 0; width:'+imageWidth+'px; height:'+imageHeight+'px" src="'+this._path + 'error/amp_pop-up_disconnected_server_reciever.png" /></div>').appendTo(this.$container);

    this.$ampErrorOverlay.css({
        'background' : this.ui.overlay.background,
        'opacity' : this.ui.overlay.opacity
    });

    this.$ampErrorWindow.css({
        'top' : (this.sizes.pageHeight/2)/2 + 'px',
        'left' : ((this.sizes.width - windowWidth)/2) + 'px',
        'border-style': 'solid',
        'border-width ': '1px',
        'border-color' : 'rgb(0, 0, 0)',
        'border-radius': '-6px',
        'background-color' : '#000',
        'box-shadow' : '0px 2px 0px 0px rgba(35, 35, 35, 0.54), inset 0px 3px 0px 0px rgba(255, 255, 255, 0.15)'
    });

    var height = 184 * this.sizes.scaleFactor, //real image height
        width = 184 * this.sizes.scaleFactor, // real image width
        left = (windowWidth - ((2*width) + (width/3)))/2,
        top = windowHeight - height;

    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/reload.png',
        'height' : height,
        'width' : width,
        'left' : left,
        'top' : top,
        'container' : this.$ampErrorWindow,
        callback : function(){
            self.removeAmpError();
            MCAPi.getHeartBeat(function(){
                amp1.init();
                if(amp2) {
                    amp2.init();
                }
            });
        }
    });

    Utils.addSimplePushGraphic({
        'img' : this._path + 'buttons/abbrechen.png',
        'height' : height,
        'width' : width,
        'top' : top,
        'left' : left + width + width/ 3,
        'container' : this.$ampErrorWindow,
        callback : function(){
            self.removeAmpError();
        }
    });
};

/**
 * Sets AMP connection error Text
 */
MCAPage.prototype.setAmpConnectionErrorText = function(){
    var text = 'Verbindung zu Verstärker 0 und 1 unterbrochen! Bitte Verstärker und Kabel prüfen!';
    this.$ampErrorWindow.find('p').html(text);
};

/**
 * Removes Amp connection error window
 */
MCAPage.prototype.removeAmpError = function(){
    this.$ampErrorOverlay.remove();
    this.$ampErrorWindow.remove();
};

/**
 * Partymode Pop up Class
 * @param page
 * @returns {PartyModePopUp}
 * @constructor
 */
PartyModePopUp = function(page) {
    this.page = page;
    this.$overlay = $('<div style="display:none; position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.page.sizes.width+'px; height:'+this.page.sizes.pageHeight+'px; text-align: center;"></div>');
    this.$popUp = $('<div style="z-index: 101; display:none; position:absolute; top:0px; left:0px;"></div>');
    /*var closeBtnHeight = 76 * this.page.sizes.scaleFactor;
    this.$closeBtn = $('<img src="images/icons/close.png" style="height: '+closeBtnHeight+'px; position:absolute; top: -'+closeBtnHeight+'px; right: -'+closeBtnHeight+'px;" />');
    this.$popUp.append(this.$closeBtn);*/

    this.$popUp.css({
        'background' : 'url("'+this.page._path + 'bg/bg_modus.png") no-repeat transparent',
        'border-radius' : '2px'
    });

    this._init();
    return this;
};

/**
 * Partymode Popup init function
 * @private
 */
PartyModePopUp.prototype._init = function(){

    var self = this;

    var imageHeight = 148 * this.page.sizes.scaleFactor,
        imageWidth = 130 * this.page.sizes.scaleFactor,
        containerWidth = 800 * this.page.sizes.scaleFactor,
        containerHeight = 300 * this.page.sizes.scaleFactor;


    this.$sceneContainer = $('<div style="text-align:center; width:'+containerWidth+'px; height: '+containerHeight*1.24+'px "></div>');
    this.$headerScenes = $('<h4 style="text-align:center; color:#fff;margin:10px 0">VOREINSTELLUNGEN</h4>').appendTo(this.$sceneContainer);

    //this.$sceneContainer.appendTo(this.$popUp);
    this.$popUp.append(this.$sceneContainer);

    this.$inputContainer = $('<div style="text-align:center; width:'+containerWidth+'px; height: '+containerHeight*0.66666+'px "></div>');
    this.$headerInputs = $('<h4 style="text-align:center; color:#fff;margin:5px 0">INPUTS</h4>').appendTo(this.$inputContainer);

    //this.$inputContainer.appendTo(this.$popUp);
    this.$popUp.append(this.$inputContainer);


    var btnImageWidth = 127 * this.page.sizes.scaleFactor * 0.8,
        btnImageHeight = 129 * this.page.sizes.scaleFactor * 0.8,
        btnTop = 200 * this.page.sizes.scaleFactor,
        nameWidth = containerWidth / 4,
        nameHeight = 40 * this.page.sizes.scaleFactor,
        nameTop = 140 * this.page.sizes.scaleFactor,
        nameLeft = (containerWidth  - 3*nameWidth)/ 4,
        popUpHeight = containerHeight*2;

    this.$sceneName1 = $('<div style="position:absolute; left: '+nameLeft + 'px; top:'+nameTop+'px; width:'+nameWidth+'px; height:'+nameHeight+'px; text-align:center; display:block; color:#fff;">'+this.page._config.scenesNames[0]+'</div>').appendTo(this.$sceneContainer);
    this.$sceneName2 = $('<div style="position:absolute; left: '+ (2*nameLeft + nameWidth) +'px; top:'+nameTop+'px; width:'+nameWidth+'px; height:'+nameHeight+'px; text-align:center; display:block; color:#fff;">'+this.page._config.scenesNames[1]+'</div>').appendTo(this.$sceneContainer);
    this.$sceneName3 = $('<div style="position:absolute; left: '+ (3*nameLeft + 2*nameWidth) + 'px; top:'+nameTop+'px; width:'+nameWidth+'px; height:'+nameHeight+'px; text-align:center; display:block; color:#fff;">'+this.page._config.scenesNames[2]+'</div>').appendTo(this.$sceneContainer);

    Utils.addSimplePushGraphic({
        'img' : this.page._path + 'buttons/scene_btn.png',
        'height' :btnImageHeight,
        'width' : btnImageWidth,
        'top' : btnTop,
        'left' : nameLeft + ((nameWidth - btnImageWidth)/2),
        'container' : this.$sceneContainer,
        callback : function(e){
            self.loadScene('scene1');
        }
    });

    Utils.addSimplePushGraphic({
        'img' : this.page._path + 'buttons/scene_btn.png',
        'height' :btnImageHeight,
        'width' : btnImageWidth,
        'padding' : '0 5px 0 5px',
        'top' : btnTop,
        'left' : (2*nameLeft + nameWidth) + ((nameWidth - btnImageWidth)/2),
        'container' : this.$sceneContainer,
        callback : function(e){
            self.loadScene('scene2');
        }
    });

    Utils.addSimplePushGraphic({
        'img' : this.page._path + 'buttons/scene_btn.png',
        'height' :btnImageHeight,
        'width' : btnImageWidth,
        'padding' : '0 5px 0 5px',
        'top' : btnTop,
        'left' : (3*nameLeft + 2*nameWidth) + ((nameWidth - btnImageWidth)/2),
        'container' : this.$sceneContainer,
        callback : function(e){
            self.loadScene('scene3');
        }
    });

    this.partyInputs = [];

    /*for(var i=0; i<this.page._config.inputs.length; i++) {
        //add inputs to party mode popup
        this.partyInputs[i] = Utils.addSimplePushGraphic({
            'img' : 'inputs/input-'+this.page._config.inputs[i]+'-off.png',
            'height' :imageHeight,
            'width' : imageWidth,
            'padding' : '0 5px 0 5px',
            'container' : this.$inputContainer,
            callback : function(e){
                self.page.showSpinner();
                amp1.powerOnZones(function(){
                    //amp2.powerOnZones(function(){
                    amp1.executeCmd('PARTY_MODE_INPUT'+i, null, false, function(){
                        //amp2.executeCmd('PARTY_MODE_INPUT1', null, false, function(){
                        //amp2.powerOffUnusedZones();
                        self.resetPartyInput(function(){
                            self.setPartyInput(i);
                        });
                        //});
                    });
                    //});
                });
            }
        });
    }*/

    //add inputs to party mode popup --> prob with loop... clean this up sometime...
    if(this.page._config.inputs[0]) {
        //console.log(this.page._path + 'inputs/input-' + this.page._config.inputs[0] + '-off.png')
        this.partyInputs[0] = Utils.addSimplePushGraphic({
            'img': this.page._path + 'inputs/input-' + this.page._config.inputs[0] + '-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputContainer,
            callback: function (e) {
                self.page.showSpinner();

                var callback = function(){
                    amp1.powerOnZones(function () {
                        amp1.executeCmd('PARTY_MODE_INPUT1', null, false, function () {
                            self.resetPartyInput(function () {
                                self.setPartyInput(0);
                            });
                        });
                    });
                }

                if(amp2) {
                    callback = function(){
                        amp1.powerOnZones(function () {
                            amp2.powerOnZones(function(){
                                amp1.executeCmd('PARTY_MODE_INPUT1', null, false, function () {
                                    amp2.executeCmd('PARTY_MODE_INPUT1', null, false, function(){
                                        amp2.powerOffUnusedZones();
                                        self.resetPartyInput(function () {
                                            self.setPartyInput(0);
                                        });
                                    });
                                });
                            });
                        });
                    }
                }
                callback();
            }
        });
    }

    //add inputs to party mode popup
    if(this.page._config.inputs[1]) {
        this.partyInputs[1] = Utils.addSimplePushGraphic({
            'img': self.page._path + 'inputs/input-' + this.page._config.inputs[1] + '-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputContainer,
            callback: function (e) {
                self.page.showSpinner();

                var callback = function() {
                    amp1.powerOnZones(function () {
                        amp1.executeCmd('PARTY_MODE_INPUT2', null, false, function () {
                            self.resetPartyInput(function () {
                                self.setPartyInput(1);
                            });
                        });
                    });
                }

                if(amp2) {
                    callback = function() {
                        amp1.powerOnZones(function () {
                            amp2.powerOnZones(function(){
                                amp1.executeCmd('PARTY_MODE_INPUT2', null, false, function () {
                                    amp2.executeCmd('PARTY_MODE_INPUT2', null, false, function(){
                                        amp2.powerOffUnusedZones();
                                        self.resetPartyInput(function () {
                                            self.setPartyInput(1);
                                        });
                                    });
                                });
                            });
                        });
                    }
                }
                callback();
            }
        });
    }

    //add inputs to party mode popup
    if(this.page._config.inputs[2]) {
        this.partyInputs[2] = Utils.addSimplePushGraphic({
            'img': self.page._path + 'inputs/input-' + this.page._config.inputs[2] + '-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputContainer,
            callback: function (e) {
                self.page.showSpinner();

                var callback = function(){
                    amp1.powerOnZones(function () {
                        amp1.executeCmd('PARTY_MODE_INPUT3', null, false, function () {
                            self.resetPartyInput(function () {
                                self.setPartyInput(2);
                            });
                        });
                    });
                }

                if(amp2) {
                    callback = function(){
                        amp1.powerOnZones(function () {
                            amp2.powerOnZones(function(){
                                amp1.executeCmd('PARTY_MODE_INPUT3', null, false, function () {
                                    amp2.executeCmd('PARTY_MODE_INPUT3', null, false, function(){
                                        amp2.powerOffUnusedZones();
                                        self.resetPartyInput(function () {
                                            self.setPartyInput(2);
                                        });
                                    });
                                });
                            });
                        });
                    }
                }

                callback();
            }
        });
    }

    //add inputs to party mode popup
    if(this.page._config.inputs[3]) {
        this.partyInputs[3] = Utils.addSimplePushGraphic({
            'img': self.page._path + 'inputs/input-' + this.page._config.inputs[3] + '-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputContainer,
            callback: function (e) {
                self.page.showSpinner();

                var callback = function(){
                    amp1.powerOnZones(function () {
                        amp1.executeCmd('PARTY_MODE_INPUT4', null, false, function () {
                            self.resetPartyInput(function () {
                                self.setPartyInput(3);
                            });
                        });
                    });
                }

                if(amp2) {
                    callback = function(){
                        amp1.powerOnZones(function () {
                            amp2.powerOnZones(function(){
                                amp1.executeCmd('PARTY_MODE_INPUT4', null, false, function () {
                                    amp2.executeCmd('PARTY_MODE_INPUT4', null, false, function(){
                                        amp2.powerOffUnusedZones();
                                        self.resetPartyInput(function () {
                                            self.setPartyInput(3);
                                        });
                                    });
                                });
                            });
                        });
                    }
                }

                callback();
            }
        });
    }

    //add inputs to party mode popup
    if(this.page._config.inputs[4]) {
        this.partyInputs[4] = Utils.addSimplePushGraphic({
            'img': self.page._path + 'inputs/input-' + this.page._config.inputs[4] + '-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputContainer,
            callback: function (e) {
                self.page.showSpinner();

                var callback = function(){
                    amp1.powerOnZones(function () {
                        amp1.executeCmd('PARTY_MODE_INPUT5', null, false, function () {
                            self.resetPartyInput(function () {
                                self.setPartyInput(4);
                            });
                        });
                    });
                };

                if(amp2) {
                    callback = function(){
                        amp1.powerOnZones(function () {
                            amp2.powerOnZones(function(){
                                amp1.executeCmd('PARTY_MODE_INPUT5', null, false, function () {
                                    amp2.executeCmd('PARTY_MODE_INPUT5', null, false, function(){
                                        amp2.powerOffUnusedZones();
                                        self.resetPartyInput(function () {
                                            self.setPartyInput(4);
                                        });
                                    });
                                });
                            });
                        });
                    }
                }

                callback();
            }
        });
    }

    /*this.$closeBtn.bind('mousedown touchstart', function(e) {
        self.page.showSpinner();
        self.hide(function(){
            amp1.updateUI();
            //amp2.updateUI();
        });
    });*/

    this.$overlay.bind('mousedown touchstart', function(e) {
        self.page.showSpinner();
        self.hide(function(){
            var callback = null;
            if(amp2) {
                callback = function(){
                    amp2.updateUI();
                }
            }
            amp1.updateUI(callback);
        });
    });

    //this.$overlay.appendTo(this.page.$container);
    //this.$popUp.appendTo(this.page.$container);

    this.page.$container.append(this.$overlay).append(this.$popUp);


    this.$overlay.css({
        'background' : this.page.ui.overlay.background,
        'opacity' : this.page.ui.overlay.opacity
    });

    this.$popUp.css({
        'width' : containerWidth + 'px',
        'height' : popUpHeight + 'px',
        'background-size': containerWidth + 'px '+ popUpHeight +'px',
        'left': ((this.page.sizes.width - containerWidth) / 2) + 'px',
        'top': ((this.page.sizes.pageHeight - popUpHeight) /2) + 'px'
    });
};

/**
 * pops up the partymode pop
 * @param amps
 */
PartyModePopUp.prototype.show = function(amps){

    //Set active Input
    this.setActivePartyInput(amps, function(){

        this.$overlay.css({
            'display' : 'block'
        });

        this.$popUp.css({
            'display' : 'block'
        });

    });
};

/**
 * Hides the partymode popup
 * @param callback
 */
PartyModePopUp.prototype.hide = function(callback){

    //Set active Input
    this.$overlay.css({
        'display' : 'none'
    });

    this.$popUp.css({
        'display' : 'none'
    });

    if(callback)
        callback();
};

/**
 * sets the active party mode input
 * @param amps
 * @param callback
 */
PartyModePopUp.prototype.setActivePartyInput = function(amps, callback){
    var inputs = [], partyInput = [];

    if (amps instanceof Array) {

        for (var i = 0, l = amps.length; i < l; i++) {
            partyInput[i] = [];
            inputs[i] = [];

            for(var n= 0, length = amps[i].inputs.length; n<length; n++ ) {

                inputs[i][n] = [];
                var zoneLength = amps[i].zones.length;
                var index = n+1;
                for(var m=0; m<zoneLength; m++) {
                    if(amps[i].zones[m]['input'+index] == 'on'){
                        inputs[i][n].push(amps[i].zones[m]['input'+index]);
                    }
                }
                if(inputs[i][n].length == amps[i].zones.length) {
                    partyInput[i] = n;
                }
            }
        }

        var input = false;
        for(var a = 1; a < partyInput.length; a++)
        {
            if(partyInput[a] === partyInput[0])
                input = partyInput[0];
        }

        if(typeof input == 'number') {
            this.setPartyInput(input);
        } else {
            this.resetPartyInput();
        }

        if(callback)
            callback.call(this);
    }
};

/**
 * Changes the partymode image
 * @param input
 */
PartyModePopUp.prototype.setPartyInput = function(input){

    var $img = $(this.partyInputs[input].html());
    $img.attr('src', $img.attr('src').replace("-off.png", "-on.png"));
    this.partyInputs[input].html($img);
    this.page.hideSpinner();
};

/**
 * Resets the partymode image
 * @param callback
 */
PartyModePopUp.prototype.resetPartyInput = function(callback){

    for(var i= 0, l=this.partyInputs.length; i<l; i++) {
        var $img = $(this.partyInputs[i].html());
        $img.attr('src', $img.attr('src').replace("-on.png", "-off.png"));
        this.partyInputs[i].html($img)
    }

    if(callback)
        callback();
};

/**
 * load a pre-defined scene from a json file
 * @param scene
 */
PartyModePopUp.prototype.loadScene = function(scene) {
    this.page.showSpinner();
    /*var url = "scenes/" + scene + ".json";
        //need timeout here otherwise spinner is not showing
        setTimeout(function(){
            $.ajax({
                url: url,
                dataType: 'json',
                success: function(response) {
                    //very slow and makes UI unresponsive, maybe find a different solution

                    var callback = amp1.executeScene(response.amp1, function(){
                        amp1.updateUI();
                    });

                    if(amp2) {
                        amp1.executeScene(response.amp1, function(){
                            amp2.executeScene(response.amp2, function(){
                                amp1.updateUI(function(){
                                    amp2.updateUI();
                                });
                            });
                        });
                    }

                    callback();
              },
                error: function(error){
                }
            });
        }, 500)*/

    var sceneAmp1, sceneAmp2;
    switch(scene){
        case "scene1":
            sceneAmp1 = this.page._config.scenes[0].amp1;
            if(amp2) {
               sceneAmp2 = this.page._config.scenes[0].amp2;
            }
            break;
        case "scene2":
            sceneAmp1 = this.page._config.scenes[1].amp1;
            if(amp2) {
                sceneAmp2 = this.page._config.scenes[1].amp2;
            }
            break;
        case "scene3":
            sceneAmp1 = this.page._config.scenes[2].amp1;
            if(amp2) {
                sceneAmp2 = this.page._config.scenes[2].amp2;
            }
            break;

    }

    var executeScene = function(){
        amp1.executeScene(sceneAmp1, function(){
            amp1.updateUI();
        });
    }

    if(amp2) {
        executeScene = function(){
            amp1.executeScene(sceneAmp1, function(){
                amp2.executeScene(sceneAmp2, function(){
                    amp1.updateUI(function(){
                        amp2.updateUI();
                    });
                });
            });
        }
    }

    executeScene();

};

/**
 * Zone Popup Class
 * @param page
 * @returns {ZonePopUp}
 * @constructor
 */
ZonePopUp = function(page) {
    var self = this;

    this.page = page;

    var popupHeight = 896 * this.page.sizes.scaleFactor,
        popupWidth = 1616 * this.page.sizes.scaleFactor;

    this.$overlay = $('<div style="display:none; position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.page.sizes.width+'px; height:'+this.page.sizes.pageHeight+'px; text-align: center;"></div>');
    this.$popUp = $('<div style="z-index: 101; display:none; position:absolute; top:0px; left:0px;"></div>');

    var inputsLeft = 806 * this.page.sizes.scaleFactor,
        inputsTop = 740 * this.page.sizes.scaleFactor;

    this.$inputs = $('<div style="z-index: 102; position:absolute; top:'+inputsTop+'px; left:'+inputsLeft+'px;"></div>');

    /*var closeBtnHeight = 76 * this.page.sizes.scaleFactor;
    this.$closeBtn = $('<img src="images/icons/close.png" style="height: '+closeBtnHeight+'px; position:absolute; top: -'+closeBtnHeight+'px; right: -'+closeBtnHeight+'px;" />');*/

    var canvasWidth = 408 * this.page.sizes.scaleFactor,
        canvasHeight = 436 * this.page.sizes.scaleFactor;
    this.canvas = '<canvas style="position:absolute;top:0;left:0" width="'+canvasWidth+'" height="'+canvasHeight+'"></canvas>';

    this.title = new Mtext('', true);

    var titleWidth = 418 * this.page.sizes.scaleFactor,
        titleHeight = 50 * this.page.sizes.scaleFactor,
        titleTop = 36 * this.page.sizes.scaleFactor;

    this.title.addToPage(this.$popUp, titleTop, 0, titleWidth, titleHeight);
    this.title.css({
        'z-index' : '105',
        'fontSize' : titleHeight * 0.75 + 'px'
    });

    var volCover = {
        'width' : 417 *  this.page.sizes.scaleFactor,
        'height' : 441 * this.page.sizes.scaleFactor
    };

    this.$volCover = $('<img src="'+ this.page._path + 'bg/circle_bg_2.png" />');
    this.$volCover.css({
        'width' : volCover.width + 'px',
        'height' : volCover.height + 'px',
        'z-index' : '102',
        'position' : 'absolute',
        'top' : '0',
        'left' : '0'
    });

    var volDiv = {
        'width' : 419 * this.page.sizes.scaleFactor,
        'height' : 50 * this.page.sizes.scaleFactor
    };

    this.$volDiv = $('<div style="color:#fff;"></div>');
    this.$volDiv.css({
        'z-index' : '106',
        'width' : volDiv.width + 'px',
        'height' : volDiv.height + 'px',
        'fontSize' : volDiv.height * 0.75 + 'px',
        'text-align' : 'center',
        'position' : 'absolute',
        'left' : 0,
        'top' : (volCover.height - volDiv.height)*0.65 + 'px'
    });

    //this.$popUp.append(this.$closeBtn)
    this.$popUp.append(this.$inputs)
        .append(this.canvas)
        .append(this.$volCover)
        .append(this.$volDiv);

    /*this.$inputs.appendTo(this.$popUp);
    $(this.canvas).appendTo(this.$popUp);
    this.$volCover.appendTo(this.$popUp);
    this.$volDiv.appendTo(this.$popUp);*/

    /*this.$closeBtn.bind('mousedown touchstart', function(e) {
        self.page.showSpinner();
        self.hide(function(){
            amp1.updateUI();
            //amp2.updateUI();
        });
    });*/

    this.$overlay.bind('mousedown touchstart', function(e) {
        self.page.showSpinner();
        self.hide(function(){
            amp1.updateUI();
            if(amp2) {
                amp2.updateUI();
            }
        });
    });

    this.page.$container.append(this.$overlay).append(this.$popUp);
    //this.$overlay.appendTo(this.page.$container);
    //this.$popUp.appendTo(this.page.$container);

    this.$overlay.css({
        'background' : this.page.ui.overlay.background,
        'opacity' : this.page.ui.overlay.opacity
    });

    this.$popUp.css({
        'width' : popupWidth + 'px',
        'height' : popupHeight + 'px',
        'left': ((this.page.sizes.width - popupWidth) /2) + 'px',
        'top': ((this.page.sizes.pageHeight - popupHeight) /2) + 'px',
        'background-color' : '#000',
        'background-image' : 'url("'+this.page._path +'bg/bg-zone-popup.png")',
        'background-size': popupWidth + 'px '+ popupHeight +'px',
        'background-repeat' : 'no-repeat',
         //'box-shadow' : '1px 1px 1px #000',
        'border-radius' : '5px'
    });

    return this;
};

/**
 * ZonePopUp Init Function
 */
ZonePopUp.prototype._init = function(zone){

    var self = this;
    this._amp = zone.tile._amp;

    var volText = {
        width: 320 * this.page.sizes.scaleFactor,
        height : 50 * this.page.sizes.scaleFactor,
        left : 428 * this.page.sizes.scaleFactor,
        top : 36 * this.page.sizes.scaleFactor
    };

    this.$volText = $('<span>VOLUME</span>').appendTo(this.$popUp);
    this.$volText.css({
        'color':'#fff',
        'text-align' : 'center',
        'width' : volText.width + 'px',
        'height' : volText.height + 'px',
        'position' : 'absolute',
        'top' : volText.top + 'px',
        'left' : volText.left + 'px',
        'fontSize' : volText.height * 0.75 + 'px'
    });

    var volSlider = {
        left : 608 * this.page.sizes.scaleFactor,
        top : 116 * this.page.sizes.scaleFactor,
        height : 700 * this.page.sizes.scaleFactor
    };

    //Add Volume Slider
    this._volSlider  = new MSliderAmp(this.page._path + '/slider/', this.page);
    this._volSlider.hasBackground(false);
    this._volSlider.indicator = true;
    this._volSlider.setVertical(true);
    this._volSlider.setRange(0,100);
    this._volSlider.step = 1;
    this._volSlider.addToPage(this.$popUp, volSlider.left, volSlider.top, volSlider.height);
    this._volSlider.setState(50);
    this._volSlider.updateCallback = function(vol){
        self.drawVolume(vol);
    };
    this._volSlider.endCallback = function(vol){
        self.page.showSpinner();
        self._amp.setVolTo(vol, zone.zone);
    };

    var bassText = {
        width: 854 * this.page.sizes.scaleFactor,
        height : 50 * this.page.sizes.scaleFactor,
        left : 762 * this.page.sizes.scaleFactor,
        top : 444 * this.page.sizes.scaleFactor
    };

    this.$bassText = $('<span>BASS</span>').appendTo(this.$popUp);
    this.$bassText.css({
        'color':'#fff',
        'text-align' : 'center',
        'width' : bassText.width + 'px',
        'height' : bassText.height + 'px',
        'position' : 'absolute',
        'top' : bassText.top + 'px',
        'left' : bassText.left + 'px',
        'fontSize' : bassText.height * 0.75 + 'px'
    });

    var bassSlider = {
        left : 896 * this.page.sizes.scaleFactor,
        width : 580 * this.page.sizes.scaleFactor,
        top : 552 * this.page.sizes.scaleFactor
    };
    //Add Bass Slider
    this._bassSlider  = new MSliderAmp(this.page._path + 'slider/', this.page);
    this._bassSlider.hasBackground(false);
    this._bassSlider.indicator = true;
    this._bassSlider.setRange(-12,12);
    this._bassSlider.step = 4;
    this._bassSlider.addToPage(this.$popUp, bassSlider.left, bassSlider.top, bassSlider.width);
    this._bassSlider.setState(0);
    this._bassSlider.endCallback = function(bass){
        self.page.showSpinner();
        self._amp.setBassTo(bass, zone.zone);
    };

    var trebleText = {
        width: 854 * this.page.sizes.scaleFactor,
        height : 50 * this.page.sizes.scaleFactor,
        left : 762 * this.page.sizes.scaleFactor,
        top : 236 * this.page.sizes.scaleFactor
    };

    this.$trebleText = $('<span>TREBLE</span>').appendTo(this.$popUp);
    this.$trebleText.css({
        'color':'#fff',
        'text-align' : 'center',
        'width' : trebleText.width + 'px',
        'height' : trebleText.height + 'px',
        'position' : 'absolute',
        'top' : trebleText.top + 'px',
        'left' : trebleText.left + 'px',
        'fontSize' : trebleText.height * 0.75 + 'px'
    });

    var trebleSlider = {
        left : 896 * this.page.sizes.scaleFactor,
        width : 580 * this.page.sizes.scaleFactor,
        top : 346 * this.page.sizes.scaleFactor

    };

    //Add Treble Slider
    this._trebleSlider  = new MSliderAmp(this.page._path + 'slider/', this.page);
    this._trebleSlider.hasBackground(false);
    this._trebleSlider.indicator = true;
    this._trebleSlider.setRange(-12,12);
    this._trebleSlider.step = 4;
    this._trebleSlider.addToPage(this.$popUp, trebleSlider.left, trebleSlider.top, trebleSlider.width);
    this._trebleSlider.setState(0);
    this._trebleSlider.endCallback = function(treble){
        self.page.showSpinner();
        self._amp.setTrebleTo(treble, zone.zone);
    };

    var balText = {
        width: 854 * this.page.sizes.scaleFactor,
        height : 50 * this.page.sizes.scaleFactor,
        left : 762 * this.page.sizes.scaleFactor,
        top : 36 * this.page.sizes.scaleFactor
    };

    this.$balText = $('<span>BALANCE</span>').appendTo(this.$popUp);
    this.$balText.css({
        'color':'#fff',
        'text-align' : 'center',
        'width' : balText.width + 'px',
        'height' : balText.height + 'px',
        'position' : 'absolute',
        'top' : balText.top + 'px',
        'left' : balText.left + 'px',
        'fontSize' : balText.height * 0.75 + 'px'
    });

    var balanceSlider = {
        left : 896 * this.page.sizes.scaleFactor,
        width : 580 * this.page.sizes.scaleFactor,
        top : 144 * this.page.sizes.scaleFactor

    };

    //Add Balance Slider
    this._balanceSlider  = new MSliderAmp(this.page._path + 'slider/', this.page);
    this._balanceSlider.hasBackground(false);
    this._balanceSlider.indicator = true;
    this._balanceSlider.setRange(-18,18);
    this._balanceSlider.step = 6;
    this._balanceSlider.addToPage(this.$popUp, balanceSlider.left, balanceSlider.top, balanceSlider.width);
    this._balanceSlider.setState(0);
    this._balanceSlider.endCallback = function(balance){
        self.page.showSpinner();
        self._amp.setBalanceTo(balance, zone.zone);
    };

    //Inputs
    var self = this;

    var inputText = {
        width: 854 * this.page.sizes.scaleFactor,
        height : 50 * this.page.sizes.scaleFactor,
        left : 762 * this.page.sizes.scaleFactor,
        top : 700 * this.page.sizes.scaleFactor
    };

    this.$inputText = $('<span>INPUT</span>').appendTo(this.$popUp);
    this.$inputText.css({
        'color':'#fff',
        'text-align' : 'center',
        'width' : inputText.width + 'px',
        'height' : inputText.height + 'px',
        'position' : 'absolute',
        'top' : inputText.top + 'px',
        'left' : inputText.left + 'px',
        'fontSize' : inputText.height * 0.75 + 'px'
    });

    var imageHeight = 148 * this.page.sizes.scaleFactor;
    var imageWidth = 130 * this.page.sizes.scaleFactor;

    this.inputs = [];
    if(this.page._config.inputs[0]) {
        //add inputs to party mode popup
        this.inputs[0] = Utils.addSimplePushGraphic({
            'img': this.page._path + 'inputs/input-'+this.page._config.inputs[0]+'-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputs,
            'params': zone,
            callback: function (e, targetZone) {
                self.page.showSpinner();
                targetZone.tile._amp.setInput(targetZone.zone, '1', function () {
                    self.setInput(targetZone);
                });
            }
        });
    }

    if(this.page._config.inputs[1]) {
        //add inputs to party mode popup
        this.inputs[1] = Utils.addSimplePushGraphic({
            'img': this.page._path + 'inputs/input-'+this.page._config.inputs[1]+'-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputs,
            'params': zone,
            callback: function (e, targetZone) {
                self.page.showSpinner();
                targetZone.tile._amp.setInput(targetZone.zone, '2', function () {
                    self.setInput(targetZone);
                });
            }
        });
    }

    if(this.page._config.inputs[2]) {
        //add inputs to party mode popup
        this.inputs[2] = Utils.addSimplePushGraphic({
            'img': this.page._path + 'inputs/input-'+this.page._config.inputs[2]+'-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputs,
            'params': zone,
            callback: function (e, targetZone) {
                self.page.showSpinner();
                targetZone.tile._amp.setInput(targetZone.zone, '3', function () {
                    self.setInput(targetZone);
                });
            }
        });
    }

    if(this.page._config.inputs[3]) {
        //add inputs to party mode popup
        this.inputs[3] = Utils.addSimplePushGraphic({
            'img': this.page._path + '/inputs/input-'+this.page._config.inputs[3]+'-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'marginTop': 0,
            'padding': '0 5px 0 5px',
            'container': this.$inputs,
            'params': zone,
            callback: function (e, targetZone) {
                self.page.showSpinner();
                targetZone.tile._amp.setInput(targetZone.zone, '4', function () {
                    self.setInput(targetZone);
                });
            }
        });
    }

    if(this.page._config.inputs[4]) {

        //add inputs to party mode popup
        this.inputs[4] = Utils.addSimplePushGraphic({
            'img': this.page._path + 'inputs/input-'+this.page._config.inputs[4]+'-off.png',
            'height': imageHeight,
            'width': imageWidth,
            'padding': '0 5px 0 5px',
            'container': this.$inputs,
            'params': zone,
            callback: function (e, targetZone) {
                self.page.showSpinner();
                targetZone.tile._amp.setInput(targetZone.zone, '5', function () {
                    self.setInput(targetZone);
                });
            }
        });
    }
    var onButtonHeight = 160 * this.page.sizes.scaleFactor,
        onButtonWidth = 305 * this.page.sizes.scaleFactor,
        onButtonLeft = 56 * this.page.sizes.scaleFactor,
        onButtonTop = 504 * this.page.sizes.scaleFactor;

    //add on/off Button
    this.onBtn = Utils.addSimplePushGraphic({
        'img' : this.page._path + 'buttons/power-off.png',
        'height' : onButtonHeight,
        'width' : onButtonWidth,
        'top' : onButtonTop,
        'left' : onButtonLeft,
        'params' : zone,
        'container' : this.$popUp,
        callback : function(e, targetZone){
            self.page.showSpinner();

            var power = 'off';
            if(self._amp.zones[targetZone.zone-1].power == 'off') {
                power = 'on';
            }

            targetZone.tile._amp.powerZone(targetZone.zone, power, true, function(data){
                self.setPower(targetZone, data[targetZone.zone-1].power);
                self.setMute(targetZone);
                self.page.hideSpinner();
            });
        }
    });


    var muteHeight = 160 * this.page.sizes.scaleFactor,
        muteWidth = 305 * this.page.sizes.scaleFactor,
        muteLeft = 56 * this.page.sizes.scaleFactor,
        muteTop = 700 * this.page.sizes.scaleFactor;

    //add mute Button
    this.muteBtn = Utils.addSimplePushGraphic({
        'img' : this.page._path + 'buttons/mute-off.png',
        'height' : muteHeight,
        'width' : muteWidth,
        'top' : muteTop,
        'left' : muteLeft,
        'container' : this.$popUp,
        'params' : zone,
         callback : function(e, targetZone){
             self.page.showSpinner();
             targetZone.tile._amp.toggleMute(targetZone.zone, function(){
                 self.setMute(targetZone, self._amp.zones[targetZone.zone-1].mute);
                 self.page.hideSpinner();
             });
         }
    });
};


/**
 * pops up the zone popup
 */
ZonePopUp.prototype.show = function(zone){

    this._init(zone);

    this.updatePopUp(zone);

    this.$overlay.css({
        'display' : 'block'
    });

    this.$popUp.css({
        'display' : 'block'
    });
};

/**
 * Hides the zone popup
 * @param callback
 */
ZonePopUp.prototype.hide = function(callback){

    this.$overlay.css({
        'display' : 'none'
    });

    this.$popUp.css({
        'display' : 'none'
    });

    if(this.$volText)
        this.$volText.remove();

    if(this._volSlider)
        this._volSlider.remove();

    if(this.$trebleText)
        this.$trebleText.remove();
    if(this._trebleSlider)
        this._trebleSlider.remove();

    if(this.$bassText)
        this.$bassText.remove();
    if(this._bassSlider)
        this._bassSlider.remove();

    if(this.$balText)
        this.$balText.remove();
    if(this._balanceSlider)
        this._balanceSlider.remove();

    if(this.onBtn)
        this.onBtn.remove();
    if(this.muteBtn)
        this.muteBtn.remove();

    this.$inputText.remove();
    for(var i= 0, l= this.inputs.length; i<l; i++) {
        if(this.inputs[i])
            this.inputs[i].remove();
    }

    if(callback)
        callback();
};

/**
 * pops up the zone popup
 */
ZonePopUp.prototype.updatePopUp = function(zone){

    //Set Zone title
    this.title.setText(zone.name);

    //set volume slider
    var vol = zone.tile._calVol(zone.volume);
    this._volSlider.setState(vol);

    //set bass slider
    this._bassSlider.setState(zone.bass);

    //set balance slider
    this._balanceSlider.setState(zone.balance);

    //set treble slider
    this._trebleSlider.setState(zone.treble);

    //Set active input
    this.setInput(zone);

    //Set Power status
    this.setPower(zone);

    //Set mute status
    this.setMute(zone);

    //draw Volume
    this.drawVolume(vol);
    //console.log(this.$popUp.find('canvas'));
};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.setInput = function(zone, number){
    var inputs = ['input1','input2','input3','input4','input5'],
        input;

    if(typeof number != 'undefined') {
        input = number;
    } else {

        for (var i = 0, l = inputs.length; i < l; i++) {
            if (zone[inputs[i]] == 'on')
                input = parseInt(inputs[i].replace('input', '')) - 1;
        }
    }

    this.resetInput(function(){

        if(this.inputs[input]) {
            var $img = $(this.inputs[input].html());
            $img.attr('src', $img.attr('src').replace("-off.png", "-on.png"));
            this.inputs[input].html($img);
            this.page.hideSpinner();
        }
        this.page.hideSpinner();
    });
};

/**
 * Resets the input image
 * @param callback
 */
ZonePopUp.prototype.resetInput = function(callback){

    for(var i= 0, l=this.inputs.length; i<l; i++) {
        var $img = $(this.inputs[i].html());
        $img.attr('src', $img.attr('src').replace("-on.png", "-off.png"));
        this.inputs[i].html($img)
    }

    if(callback)
        callback.call(this);
};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.setPower = function(zone, power){
    var self = this,
        setting = zone.power;

    if(power)
        setting = power;

    this.resetPower(function(){
        if(setting == 'on') {
            var $img = $(this.onBtn.html());
            $img.attr('src', $img.attr('src').replace("power-off", "power-on"));
            this.onBtn.html($img);
        }
    });
};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.resetPower = function(callback){

    var $img = $(this.onBtn.html());
    $img.attr('src', $img.attr('src').replace("power-on", "power-off"));
    this.onBtn.html($img);

    if(callback)
        callback.call(this);

};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.setMute = function(zone, mute){
    var setting = zone.mute;

    if(mute) {
        setting = mute;
    }

    this.resetMute(function(){
        if(setting == 'on') {
            var $img = $(this.muteBtn.html());
            $img.attr('src', $img.attr('src').replace("mute-off", "mute-on"));
            this.muteBtn.html($img);
        }
    });
};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.resetMute = function(callback){

    var $img = $(this.muteBtn.html());
    $img.attr('src', $img.attr('src').replace("mute-on", "mute-off"));
    this.muteBtn.html($img);

    if(callback)
        callback.call(this);

};

/**
 * Changes the input image
 * @param input
 */
ZonePopUp.prototype.drawVolume = function(volume){

    this.$volDiv.html(volume + ' %');

    var canvas = this.$popUp.find('canvas')[0],
        context = canvas.getContext('2d'),
        x = canvas.width / 1.95,
        y = canvas.height / 1.58,
        startAngle = 0.66,
        endAngle = 0.66;

    if(this.page.sizes.scaleFactor == 1) {
        var radius = canvas.width / 3 - 21,
            counterClockwise = false,
            lineWidth = canvas.width * 0.067;
    } else if(this.page.sizes.scaleFactor == 0.5){
        var radius = canvas.width / 3 - 12,
            counterClockwise = false,
            lineWidth = canvas.width * 0.067;
    }

    //clear canvas
    context.clearRect (0, 0, canvas.width, canvas.height );

    //Indicator
    //if vol > 80 indicate dangerzone
    if(volume < this._amp._page.ui.config.dangerZone) {
        var startAngleInd = startAngle * Math.PI;
        var endAngleInd = endAngle * Math.PI;
        if(volume != 0){
            endAngleInd = ((((2.34-0.66)/100)*volume)+endAngle) * Math.PI;
        }

        context.beginPath();
        context.arc(x, y, radius, startAngleInd, endAngleInd, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#ffffff';
        context.stroke();

        // Dark grey rest
        var endAngleInd2 = ((((2.34-0.66)/100)*100)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd, endAngleInd2, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#333';
        context.stroke();
    } else {
        //console.log(volume)
        var startAngleInd = startAngle * Math.PI;
        var endAngleInd = endAngle * Math.PI;
        if(volume != 0){
            endAngleInd = ((((2.34-0.66)/100)*this._amp._page.ui.config.dangerZone)+endAngle) * Math.PI;
        }

        context.beginPath();
        context.arc(x, y, radius, startAngleInd, endAngleInd, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#ffffff';
        context.stroke();

        //Danger Zone
        var endAngleInd2 = ((((2.34-0.66)/100)*volume)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd, endAngleInd2, counterClockwise);
        context.lineWidth = lineWidth;
        context.strokeStyle = '#FF0000';
        context.stroke();

        // Dark grey rest
        var endAngleInd3 = ((((2.34-0.66)/100)*100)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd2, endAngleInd3, counterClockwise);
        context.lineWidth = lineWidth;
        context.strokeStyle = '#333';
        context.stroke();
    }
};

/**
 * Raspberry PI Server Class
 * @constructor
 * @param ip
 * @param page
 */
MCAServer = function(ip, page){
    this.page = page;
    this.api = "http://"+ip +"/peripheral/mca";
};

/**
 * Get a heart beat from the PI
 * @param callback
 */
MCAServer.prototype.getHeartBeat = function(callback){
    var self = this;

    //$.ajaxSetup({async:false});
    $.ajax({
        url : this.api,
        //async:false,
        type : "POST",
        timeout: 3000,
        contentType: "application/json",
        success: function(data) {

            if(callback)
                callback.call(self)
        },
        error: function(error){
            self.page.showPIError();
        }
    });
};

/**
 * Initializes the amplifiers
 */
MCAServer.prototype.initAmps = function(config){

    this.config = config;
    var inputs = [this.page._path +'inputs/nas.png', this.page._path +'inputs/tv.png', this.page._path +'inputs/airplay.png', this.page._path +'inputs/sky.png', this.page._path +'inputs/hifi.png'];

    if(this.config.inputs) {
        for (var i=0; i<this.config.inputs.length; i++) {
            inputs[i] = this.page._path + 'inputs/' + this.config.inputs[i] + '.png';
        }
    }

    var zoneNamesAmp1 = [];
    for(var i=0; i<this.config.zones[0]; i++) {
        zoneNamesAmp1.push(this.config.names[i]);
    }

    if(this.config.amps.length != 0) {
        window.amp1 = new Amp({
            'deviceid': '0',
            'zones': this.config.zones[0],
            'maxZones': 6,
            'api': MCAPi.api,
            'volMax': 0,
            'volMin': -61,
            'inputs': inputs,
            'volLimit': 50,
            'volFactor': 3,
            'zoneNames': zoneNamesAmp1 //['Küche', 'Essen', 'Wohnen','Flur', 'Büro', 'Schlafen']
        }, this.page);

    }

    if(this.config.amps.length == 2) {
        var zoneNamesAmp2 = [];
        for(var i=0; i<this.config.zones[1]; i++) {
            zoneNamesAmp2.push(this.config.names[i + this.config.zones[0]]);
        }

        window.amp2 = new Amp({
         'deviceid' : '1',
         'zones' :4,
         'maxZones' : 6,
         'api' : MCAPi.api,
         'volMax' : 0,
         'volMin' : -61,
         'volLimit' : 50,
         'volFactor' : 3,
         'inputs' : inputs,
         'zoneNames' : zoneNamesAmp2 //['Bad', 'Kind 1', 'Kind 2', 'Gästebad']
         }, this.page);
    } else {
        window.amp2 = null;
    }
};

/**
 * MCA-66 Amplifier Class
 * @param object
 * @param page
 * @constructor
 */
Amp = function(object ,page){
    this._page = page;
    this.api = object.api;
    this._amtZones = object.zones;
    this._maxZones = object.maxZones;
    this.deviceid = object.deviceid;
    this.volMax = object.volMax;
    this.volMin = object.volMin;
    this.volLimit = object.volLimit || 50;
    this.volFactor = object.volFactor || 3;
    this.inputs = object.inputs;
    this.zoneNames = object.zoneNames;
    this._amtZones = (typeof this._amtZones) == "undefined"  ? 6 : this._amtZones;

    this.init();
};

/**
 * Initializes the Amplifier and creates the AMP Tiles
 */
Amp.prototype.init = function(callback){
    var self = this;

    //Remove all tiles from DOM in case they already exist
    if(typeof this.zones != 'undefined') {
        for(var k= 0, l=this.zones.length; k<l; k++) {
            this.zones[k].tile.removeTile();
        }
    }

    this.zones = []; //amtZones properties

    //init Zones
    for(var i =0; i<this._amtZones;i++) {
        this.zones[i] = {
            'div': {},
            'name' : this.zoneNames[i]
        }
    }

    //update states and create Volume Tiles
    this.updateStates(function(){

        for(var j= 0, l = self.zones.length; j<l; j++) {
            if(!self.zones[j].tile)
            self.zones[j].tile = new AMPTile(j, self, self._page, self._page.sizes.width/self._page.ui.config.tilesPerRow, self._page.sizes.heightTilesContainer/self._page.ui.config.rows);
        }
        self._page.hideSpinner();

        if(callback)
            callback();
    });
};

/**
 * Toggles mute for a zone
 * @param zone
 * @param callback
 */
Amp.prototype.toggleMute = function(zone , callback){
    var self = this;

    var mute = 'off';
    if(this.zones[zone-1].mute == 'off')
        mute = 'on';

    this._page.$popUpZone.setMute(zone, mute);

    this.executeCmd('MUTE_ON_OFF', zone , false, function(){
        self.updateStates(function(data){
            if(data) {
                if (callback)
                    callback();
            }
        });
    });
};

/**
 * Sets the input for a specific zone
 * @param zone
 * @param input
 * @param callback
 */
Amp.prototype.setInput = function(zone , input, queue, callback){
    var self = this,
        cmd = 'INPUT' + input;

    this._page.$popUpZone.setInput(zone, input-1);

    this.executeCmd(cmd, zone , false, function(){
        self.updateStates(function(data){
            if(data) {
                if (callback)
                    callback();
            }
        });
    });
};

/**
 * Power on or off a specific zone
 * @param zone
 * @param power
 * @param callback
 */
Amp.prototype.powerZone = function(zone, power, popup, callback){
    var self = this,
        cmd = 'POWER_OFF',
        setting = 'off',
        queue = true;

    if(power == 'on') {
        cmd = 'POWER_ON';
        setting = 'on';
    }

    if(popup) {
        this._page.$popUpZone.setPower(zone, setting);
        queue = false;
    }

    if(queue) {
        this.zones[zone-1].power = setting;
    }

    this.executeCmd(cmd, zone, queue, function(data){
        if(data) {
            self.updateStates(function(data){
                 if(callback)
                    callback(data);
            });
        }
    });
};

/**
 * Scene to mute all zones
 * @param mute on/off
 */
Amp.prototype.muteAllZones = function(mute){
    var self = this;
    var zones = [];

    for(var i= 1; i <= this._amtZones;i++)
    {
        zones.push({
            'zone' : i,
            'mute' : mute,
            'power' : 'on'
        });
    }

    this.executeScene(zones, function(data){
        self.updateUI(function(){
        });
    })
};

/**
 * Switches on all zones
 * @param callback
 */
Amp.prototype.powerOnZones = function(callback){

    var self = this;

    this.executeCmd('POWER_ALL_ON', null, false, function(data){
        setTimeout(function(){
            self.updateUI(function(){
                self.powerOffUnusedZones();
                if(callback)
                    callback();
                self._page.hideSpinner();
            });
        },1000);
    })
};

/**
 * power off all unsed zones
 */
Amp.prototype.powerOffUnusedZones = function(){

    var zones = this._amtZones;
    if(zones < this._maxZones) {
        while(zones <= this._maxZones) {
            zones++;
            this.executeCmd('POWER_OFF', zones, false, function(){ });
        }
    }
};

/**
 * Switch all zones off
 */
Amp.prototype.powerOffZones = function(){
    var self = this;
    this.executeCmd('POWER_ALL_OFF', null, false, function(data){
        self.updateUI();
    });
};

/**
 * All Zones Volume up
 * @param delta - delta by how much to increase the volume of each zone
 * @param callback
 */
Amp.prototype.allVolUp = function(delta, callback){
    var self = this;
    var i = 0;

    function volUp(delta){
        if(i<self._amtZones) {

            //value based on 100%
            var calcDelta = -1*self.volMin + self._calVolToAmp(delta);

            calcDelta = calcDelta + self.zones[i].volume;

            if(calcDelta > self.volMax)
                calcDelta = self.volMax;
            else if(calcDelta < self.volMin)
                calcDelta = self.volMin;

            var zones = [{
                'zone' : self.zones[i].zone,
                'volume' : calcDelta
            }];

            self.executeScene(zones, function(){
                //console.log(i)
                if(i==self._amtZones -1) {
                    if (callback)
                        callback();
                } else {
                    i++;
                    volUp(delta);
                }
            });
        }
    }
    volUp(delta);
};

/**
 * All Zones Volume Down
 * @param delta
 * @param callback
 */
Amp.prototype.allVolDown = function(delta, callback){
    var self = this;
    var i = 0;

    function volDown(delta){
        if(i<self._amtZones) {

            //value based on 100%
            var setVol = -1*delta;

            var calcDelta = -1 * (-1*self.volMin + self._calVolToAmp(setVol));

            calcDelta = calcDelta + self.zones[i].volume;

            if(calcDelta > self.volMax)
                calcDelta = self.volMax;
            else if(calcDelta < self.volMin)
                calcDelta = self.volMin;

             var zones = [{
                 'zone' : self.zones[i].zone,
                 'volume' : calcDelta
             }];

             self.executeScene(zones, function(){
                    //console.log(i)
                    if(i==self._amtZones -1) {
                        if (callback)
                            callback();
                    } else {
                        i++;
                            volDown(delta);
                    }
             });
        }
    }
    volDown(delta);
};

/**
 * Sets bass for a specific zone
 * @param bass - possible values: -12, -8, -4, 0, 4, 8, 12
 * @param zone
 */
Amp.prototype.setBassTo = function(bass, zone){
    var self = this;
    //only the following for bass are allowed
    var values = [-12, -8,-4, 0, 4, 8, 12];
    if(values.indexOf(bass) == -1) {
        var closest = null;
        $.each(values, function (index, value) {
            if (closest == null || Math.abs(value - bass) < Math.abs(closest - bass)) {
                closest = value;
            }
        });
        bass = closest;
    }

    if(!bass)
        bass = 0;

    var zones = [{
        'zone' : zone,
        'bass' : bass
    }];

    this.executeScene(zones , function(data){
        self._page.hideSpinner();
    })
};

/**
 * Sets treble to a specific value
 * @param treble - possible values: -12, -8, -4, 0, 4, 8, 12
 * @param zone
 */
Amp.prototype.setTrebleTo = function(treble, zone){
    var self = this;
    //only the following for bass are allowed
    var values = [-12, -8,-4, 0, 4, 8, 12];
    if(values.indexOf(treble) == -1) {
        var closest = null;
        $.each(values, function (index, value) {
            if (closest == null || Math.abs(value - treble) < Math.abs(closest - treble)) {
                closest = value;
            }
        });
        treble = closest;
    }

    if(!treble)
        treble = 0;

    var zones = [{
        'zone' : zone,
        'treble' : treble
    }];

    this.executeScene(zones , function(data){
        self._page.hideSpinner();
        //self.updateUI();
    })
};

/**
 * Updates the UI
 * @param callback
 */
Amp.prototype.updateUI = function(callback){

    var self = this;
    //setTimeout(function(){

        self.updateStates(function(states){
            if(states && states.length > 0) {

                for (var i = 0; i < self._amtZones; i++) {
                    var power = states[i].power,
                        mute = states[i].mute,
                        vol = self.zones[i].tile._calVol(states[i].volume),
                        input = self.getActiveInput(states[i]);

                    self.zones[i].tile.setPowerStatus(power);
                    self.zones[i].tile.setInput(input);

                    if (mute == 'on') {
                        self.zones[i].tile.setMute(mute);
                    } else {
                        self.zones[i].tile.setVolume(vol);
                    }
                }
            }

            if(callback) {
                callback();
            } else {
                self._page.hideSpinner();
            }
        });
    //},500);
};

/**
 * Gets the active input of a zone
 * @param states
 * @returns {string}
 */
Amp.prototype.getActiveInput = function(states){
    var inputs = this.inputs.length,
        zones = this._amtZones,
        activeInput = "";

    for(var i=0; i<inputs;i++) {
        var index = i+1;
        if(states['input' + index] == 'on') {
            activeInput = i;
            break;
        }
    }

    return activeInput;
};

/**
 * Sets balance of a specific zone
 * @param balance - allowed values: -18, -12,-6, 0, 6, 12, 18
 * @param zone
 */
Amp.prototype.setBalanceTo = function(balance, zone){
    var self = this;
    //only the following for bass are allowed
    var values = [-18, -12,-6, 0, 6, 12, 18];
    if(values.indexOf(balance) == -1) {
        var closest = null;
        $.each(values, function (index, value) {
            if (closest == null || Math.abs(value - balance) < Math.abs(closest - balance)) {
                closest = value;
            }
        });
        balance = closest;
    }

    if(!balance)
        balance = 0;

    var zones = [{
        'zone' : zone,
        'balance' : balance
    }];

    this.executeScene(zones, function(data){
        self._page.hideSpinner();
        //self.updateUI();
    })
};

/**
 * Updates states
 * @param callback
 */
Amp.prototype.updateStates = function(callback){

    var self = this;
    this.getStates(function(states){
        if(states) {
            for (var i = 0; i < self._amtZones; i++) {
                $.extend(self.zones[i], states[i]);
            }
        }
        if (callback)
            callback(states);
    });
};

/**
 * Gets states from the PI
 * @param callback
 */
Amp.prototype.getStates = function(callback){

    var self = this;

    //$.ajaxSetup({async:false});
    $.ajax({
        url : this.api,
        type : "POST",
        //async:false,
        data: JSON.stringify({
            deviceid : this.deviceid,
            type : 'state'

        }),
        contentType: "application/json",
        success: function(data) {
            var arr = [];
            try {
                var res = $.parseJSON(data.replace("{XC_SUC}", ""));
            } catch(e){
                if (callback)
                    callback(null);

                return;
            }

            for(var i= 0, length = res.length; i < length; i++) {
                if(res[i].zone != 0)
                    arr.push(res[i]);
            }
            arr.sort(function(a,b) { return parseInt(a.zone) - parseInt(b.zone) });

            if(arr.length == 0) {
                if(typeof self._page.$ampErrorWindow != 'undefined') {

                    /*if (self._page.$ampErrorWindow.is(':visible')) {
                        self._page.setAmpConnectionErrorText();
                    } else {
                        self._page.showAmpConnectionError(self.deviceid);
                    }*/

                } else {
                    self._page.showAmpConnectionError(self.deviceid);
                }
            }

            if (callback)
                callback(arr);

        },
        error: function(error){
        }
    });

};

/**
 * Sets volume for a specific zone
 * @param vol
 * @param zone
 * @param callback
 */
Amp.prototype.setVolTo = function(vol, zone, callback){
    var self = this;

    //value based on 100%
    var setVol = this._calVolToAmp(vol);

    var zones = [{
        'zone' : zone,
        'volume' : setVol
    }];

    this.executeScene(zones, function(){
        //self.updateUI(function(){
            if(callback)
                callback();
            self._page.hideSpinner();
        //});
    });
};

/**
 * calculates 100% value to Amplifier volume
 * @param vol
 * @returns {*}
 * @private
 */
Amp.prototype._calVolToAmp = function(vol) {

    if(vol == 0 || vol < 0)
        return this.volMin;
    else if(vol == 100 || vol > 100)
        return this.volMax;

    //0 - 50 calculate forst two third of the vol
    else if(vol <= this.volLimit) {
        var limitButtom = this.volMin,
            limitTop = this.volMin/this.volFactor;

        return -1*(Math.round( ((limitButtom - limitTop)/this.volLimit) * vol - limitButtom));
    }
    //50 - 100 last third of the vol
    else if(vol > this.volLimit) {
        var limitBottom = this.volMin/this.volFactor;
            limitTop = this.volMax;

        return -1*(Math.round((((limitBottom - limitTop)/this.volLimit) * (vol - this.volLimit)) - limitBottom));
    }
    /*else
        return Math.floor(-1*((this.volMin /100 * vol) - this.volMin)); */
};

/**
 * Executes an amp command
 * @param cmd
 * @param zone
 * @param callback
 */
Amp.prototype.executeCmd = function(cmd, zone, queue, callback){

    if(!cmd) {
        return;
    }

    zone = zone ? zone : "1";

    //$.ajaxSetup({async:true});
    if(!queue) {
        $.ajax({
            url: this.api,
            type: "POST",
            //async:true,
            data: JSON.stringify({
                type: 'control',
                zone: zone,
                command: cmd,
                deviceid: this.deviceid
            }),
            contentType: "application/json",
            success: function (data) {
                var res = null;
                try {
                    res = JSON.parse(data.replace('{XC_SUC}', ''))
                } catch(e){}

                if (callback)
                    callback(res);

            }
        });
    } else {
        var params = {
            url: this.api,
            type: "POST",
            data: JSON.stringify({
                type: 'control',
                zone: zone,
                command: cmd,
                deviceid: this.deviceid
            }),
            contentType: "application/json",
            success: function (data) {
                var res = null;
                try {
                    res = JSON.parse(data.replace('{XC_SUC}', ''))
                } catch(e){}

                if (callback)
                    callback(res);

            }
        };
        MCAAjaxManager.addReq(params);
    }
};

/**
 * Executes a scene
 * @param zones
 * @param callback
 */
Amp.prototype.executeScene = function(zones, callback){
    var self = this;

    if(!zones) {
        return;
    }

    //$.ajaxSetup({async:true});
    $.ajax({
        url : this.api,
        type : "POST",
        async: true,
        data: JSON.stringify({
            deviceid: this.deviceid,
            power : "on",
            mute : "off",
            type : 'scene',
            zones : zones
        }),
        contentType: "application/json",
        success: function(data) {
            //self.updateStates();
            var success = true;
            if(data.indexOf('{XC_SUC}') != -1) {
                if(callback)
                    callback(success);
            } else {
                success = false;
                if(callback)
                    callback(success);
            }
        }
    });
};

/**
 * Amp Tile Class
 * @param zone
 * @param amplifier
 * @param page
 * @param width
 * @param height
 * @returns {AMPTile}
 * @constructor
 */
AMPTile = function(zone, amplifier, page, width, height){
    this._page = page;
    this._zone = zone;
    this._amp = amplifier;
    this.width = width;
    this.height = height;
    this.inputPopUp = null;
    this.createTile();
    return this;
};

/**
 * remove a tile from the DOM
 */
AMPTile.prototype.removeTile = function(){
    this._amp.zones[this._zone].div.remove();
};

/**
 * Creates a tile and draws the canvas
 * @param width
 * @param height
 */
AMPTile.prototype.createTile = function(){

    this._amp.zones[this._zone].div = $('<div style="position:relative; width:'+this.width+'px; height:'+this.height+'px;float:left;"><canvas width="'+this.width+'" height="'+this.height+'"></canvas></div>');

    var canvas = this._amp.zones[this._zone].div.find('canvas')[0];
    var vol = this._calVol(this._amp.zones[this._zone].volume);
    var power = this._amp.zones[this._zone].power;
    var name = this._amp.zones[this._zone].name;
    var mute = this._amp.zones[this._zone].mute;

    var inputs = this._amp.inputs.length,
        input = 'off';

    for(var i =0; i<inputs; i++) {
        if(this._amp.zones[this._zone]['input'+(i+1)] == 'on')
            input = i;
    }

    this.drawCanvas(canvas, vol, power, name, input, mute);

    this._amp._page.$tilesContainer.append(this._amp.zones[this._zone].div);
    //$(this._amp.zones[this._zone].div).appendTo(this._amp._page.$tilesContainer);
};

/**
 * Draws the volume canvas
 * @param canvas
 * @param volume
 * @param power
 * @param name
 * @param input
 * @param mute
 */
AMPTile.prototype.drawCanvas = function(canvas, volume, power, name, input, mute){

    if(volume || volume == 0) {
        this.drawVolumeStatus(canvas, volume);
        this.addVolume(volume)
    }

    if(mute == "on") {
        var muteHeight = 72 * this._page.sizes.scaleFactor;
        this._volDiv.html('<img style="height:' + muteHeight + 'px;" src="'+this._page._path +'icons/mute.png" />');
    }

    this.addCoverImage();

    this.addPowerStatus(power);

    this.addZoneName(name);

    this.addInput(input);

};

/**
 * Calculates the volume coming from the Amp to 100% value
 * @param vol
 * @returns {number}
 * @private
 */
AMPTile.prototype._calVol = function(vol) {
    var factor = this._amp.volMin / this._amp.volFactor;

    if(vol == this._amp.volMax || vol > this._amp.volMax)
        return 100;
    else if(vol == this._amp.volMin || vol < this._amp.volMin)
        return 0;
    else if((vol > this._amp.volMin) && (vol <= this._amp.volMin / this._amp.volFactor)) {
        return Math.round((this._amp.volLimit / (this._amp.volMin - factor)) * (this._amp.volMin - vol));
    }
    else if(vol > (this._amp.volMin / this._amp.volFactor)) {
        return Math.round((this._amp.volLimit / factor) * (this._amp.volMin - vol) - this._amp.volLimit);
    }
    //return -1*(Math.round(100 / this._amp.volMin*vol)) +100;
};

/**
 * Adds the power status to the tile
 * @param power
 */
AMPTile.prototype.addPowerStatus = function(power){
    var self = this;

    if(power != 'on' && power != 'off')
        power = 'off';

    //console.log(this._amp.zones[this._zone].div.width());
    var width = this.width/2;//this._amp.zones[this._zone].div.width()/2;
    var height = this.height/8; //this._amp.zones[this._zone].div.height()/8;

    var imageheight = 40 * this._page.sizes.scaleFactor;
    this.powerDiv = $('<div class="power" style="z-index:50; color: white; text-align:center; position:absolute; bottom:'+ (height - imageheight)+'px; left:'+width/2+'px; font-size:'+height*0.4+'px; width:'+width+'px; height:'+height+'px"><img style="margin-top:'+height*0.25+'px; height:'+imageheight+'px" src="'+this._page._path+'icons/'+power+'.png" /></div>');

    this.powerDiv.bind('mousedown touchstart', function(evt){
        evt.preventDefault();
        //self._page.showSpinner();

        var power = 'on';
        if(self._amp.zones[self._zone].power == 'on')
            power = 'off';

        self.setPowerStatus(power);

        self._amp.powerZone(self._amp.zones[self._zone].zone, power, false) /*, function(){
            setTimeout(function(){
                //self._amp.updateUI(function(){
                    //self._page.hideSpinner();
                //});
            }, 500);
        });*/
    });

    this._amp.zones[this._zone].div.append(this.powerDiv);
    //$(this.powerDiv).appendTo(this._amp.zones[this._zone].div);
};

/**
 * sets the power status to the tile
 * @param power
 */
AMPTile.prototype.setPowerStatus = function(power){
    this.powerDiv.find('img').attr('src', this._page._path +'icons/'+power+'.png')
};

/**
 * Set the zone power status
 * @param power
 */
AMPTile.prototype.setPowerStatus = function(power){
    this._amp.zones[this._zone].tile.powerDiv.find('img').attr('src', this._page._path +'icons/'+power+'.png');
};

/**
 * Puts the cover image over the volume circle
 */
AMPTile.prototype.addCoverImage = function(){
    var self = this;

    var top = 198;
    var left = 70;

    if( this._page.sizes.scaleFactor == 0.5) {
        top = top/2;
        left = left/2;
    }

    var imageHeight = 260 * this._page.sizes.scaleFactor;
    this.coverImage = $('<img style="position:absolute; top:'+top+'px; left:'+left+'px; height : '+imageHeight+'px" src="'+this._page._path +'volCover.png" />');

    this.coverImage.bind('mousedown touchstart', function(evt){
        evt.preventDefault();

        self._page.$popUpZone.show(self._amp.zones[self._zone]);
    });

    //this._amp.zones[this._zone].div.append(this.coverImage);
    this.coverImage.appendTo(this._amp.zones[this._zone].div);
};

/**
 * Addes the zone name
 * @param name
 */
AMPTile.prototype.addZoneName = function(name){
    //console.log(this._amp.zones[this._zone].div.width());
    //var width = this._amp.zones[this._zone].div.width()-20;
    //var height = this._amp.zones[this._zone].div.height()/6 * 0.44;
    var width = this.width-20;
    var height = (this.height/6) * 0.44;
    this.zoneNameDiv = $('<div class="zone-name" style="text-transform: uppercase; color: white; text-overflow:ellipsis; text-align:center; position:absolute; top:'+2.5*height+'px; left:10px; font-size:'+height+'px; width:'+width+'px; height:'+height+'px">'+name+'</div>');
    //this._amp.zones[this._zone].div.append(this.zoneNameDiv);
    this.zoneNameDiv.appendTo(this._amp.zones[this._zone].div);
};

/**
 * Adds the volume
 * @param vol
 */
AMPTile.prototype.addVolume = function(vol){
    //console.log(this._amp.zones[this._zone].div.width());
    var width = this.width/4;
    var height = (this.height/8)*0.7;
    var top = (this.height - height) / 1.5;
    var left = (this.width - width) / 2;
    this._volDiv = $('<div class="volume" style="color: white; text-overflow:ellipsis; vertical-align:center; text-align:center; position:absolute; top:'+top+'px; left:'+left+'px; font-size:'+height+'px; width:'+width+'px; height:'+height+'px">'+vol+'<span style="font-size:'+height *0.6+'px;">&nbsp;&nbsp;%</span></div>');
    //this._amp.zones[this._zone].div.append(this._volDiv);
    this._volDiv.appendTo(this._amp.zones[this._zone].div);
};

/**
 * Sets the volume status
 * @param volume
 */
AMPTile.prototype.setVolume = function(volume){
    this.drawVolumeStatus(this._amp.zones[this._zone].div.find('canvas')[0], volume);
    var height = (this.height/8)*0.7;
    this._amp.zones[this._zone].tile._volDiv.html(volume+'<span style="font-size:'+height *0.6+'px;">&nbsp;&nbsp;%</span>');
};

/**
 * the set mute status
 */
AMPTile.prototype.setMute = function() {
    var muteHeight = 72 * this._page.sizes.scaleFactor;
    //console.log(this._amp.zones[this._zone])
    this._amp.zones[this._zone].tile._volDiv.html('<img style="height:' + muteHeight + 'px;" src="'+this._page._path +'icons/mute.png" />');
};

/**
 * Addes the Input
 * @param input
 */
AMPTile.prototype.addInput = function(input){
    var img = this._page._path + 'inputs/off.png',
        imageHeight = 50 * this._page.sizes.scaleFactor,
        self = this;

    if(input != 'off' && typeof input == 'number') {
        img = this._amp.inputs[input]
    }

    //var width = this._amp.zones[this._zone].div.width();
    var width = this.width;
    //var height = this._amp.zones[this._zone].div.height()/6;
    var height = this.height/6;
    this.inputDiv = $('<div class="input" style="color: white; text-overflow:ellipsis; vertical-align:center; text-align:center; position:absolute; top:'+ 4 * height *0.44 +'px; left:0px; width:'+width+'px; height:'+height+'px"></div>');
    this.inputImg = $('<img style="height : '+imageHeight+'px" src="'+img +'" />').appendTo(this.inputDiv);

    this.inputImg.bind('mouseup touchend', function(e) {
        //console.log(e),
        if(!self._page.inputPopVisible) {
            self._page.inputPopVisible = true;
            var $popUpInput = new AmpInputPopup(self._page, self._amp.zones[self._zone].div, self);
            $popUpInput.show(e, self._zone, self._amp);
        }
    });

    this._amp.zones[this._zone].div.append(this.inputDiv);
    //this.inputDiv.appendTo(this._amp.zones[this._zone].div);
};

/**
 * Sets the input status
 * @param input
 */
AMPTile.prototype.setInput = function(input){
    var self = this;

    var img = this._page._path + 'inputs/off.png';
    var imageHeight = 50 * this._page.sizes.scaleFactor;

    if(input != 'off' && typeof input == 'number') {
        img = this._amp.inputs[input]
    }

    var $img = $('<img style="height : '+imageHeight+'px" src="'+ img +'" />');

    //$img.bind('mouseup touchend', function(e) {
    $img.bind('mouseup touchend', function(e) {
        if(!self._page.inputPopVisible) {
            self._page.inputPopVisible = true;
            var $popUpInput = new AmpInputPopup(self._page, self._amp.zones[self._zone].div, self);
            $popUpInput.show(e, self._zone, self._amp);
        }
    });

    this._amp.zones[this._zone].tile.inputDiv.html($img);
};

/**
 * Draws the volume status
 * @param canvas
 * @param volume
 */
AMPTile.prototype.drawVolumeStatus = function(canvas, volume){
    var  context = canvas.getContext('2d'),
         x = canvas.width / 2,
         y = canvas.height / 1.5,
         startAngle = 0.66,
         endAngle = 0.66;

    if(this._page.sizes.scaleFactor == 1) {
        var radius = canvas.width / 3 - 21,
            counterClockwise = false,
            lineWidth = canvas.width * 0.067;
    } else if(this._page.sizes.scaleFactor == 0.5){
        var radius = canvas.width / 3 - 10,
            counterClockwise = false,
            lineWidth = canvas.width * 0.065;
    }

    //clear canvas
    context.clearRect (0, 0, canvas.width, canvas.height );

    //Indicator
    //if vol > 80 indicate dangerzone
    if(volume < this._amp._page.ui.config.dangerZone) {
        var startAngleInd = startAngle * Math.PI;
        var endAngleInd = endAngle * Math.PI;
        if(volume != 0){
            endAngleInd = ((((2.34-0.66)/100)*volume)+endAngle) * Math.PI;
        }

        context.beginPath();
        context.arc(x, y, radius, startAngleInd, endAngleInd, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#ffffff';
        context.stroke();

        // Dark grey rest
        var endAngleInd2 = ((((2.34-0.66)/100)*100)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd, endAngleInd2, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#333';
        context.stroke();
    } else {
        //console.log(volume)
        var startAngleInd = startAngle * Math.PI;
        var endAngleInd = endAngle * Math.PI;
        if(volume != 0){
            endAngleInd = ((((2.34-0.66)/100)*this._amp._page.ui.config.dangerZone)+endAngle) * Math.PI;
        }

        context.beginPath();
        context.arc(x, y, radius, startAngleInd, endAngleInd, counterClockwise);
        context.lineWidth = lineWidth;
        //context.lineCap = 'round';
        // line color
        //context.strokeStyle = '#819FF7';
        context.strokeStyle = '#ffffff';
        context.stroke();

        //Danger Zone
        var endAngleInd2 = ((((2.34-0.66)/100)*volume)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd, endAngleInd2, counterClockwise);
        context.lineWidth = lineWidth;
        context.strokeStyle = '#FF0000';
        context.stroke();

        // Dark grey rest
        var endAngleInd3 = ((((2.34-0.66)/100)*100)+endAngle) * Math.PI;
        context.beginPath();
        context.arc(x, y, radius, endAngleInd2, endAngleInd3, counterClockwise);
        context.lineWidth = lineWidth;
        context.strokeStyle = '#333';
        context.stroke();
    }
};

AmpInputPopup = function(page, container, tile){

    var self = this;
    this.page = page;
    this.tile = tile;

    this.container = container;

    var popupHeight = 200 * this.page.sizes.scaleFactor,
        popupWidth = 800 * this.page.sizes.scaleFactor;

    this.$overlay = $('<div style="display:none; position: absolute; left: 0px; top: 0px; z-index: 100; width:'+this.page.sizes.width+'px; height:'+this.page.sizes.pageHeight+'px; text-align: center;"></div>');
    this.$popUp = $('<div style="z-index: 101; display:none; position:absolute; top:0px; left:0px;"></div>');

    /* var closeBtnHeight = 76 * this.page.sizes.scaleFactor;
    this.$closeBtn = $('<img src="images/icons/close.png" style="height: '+closeBtnHeight+'px; position:absolute; top: -'+closeBtnHeight+'px; right: -'+closeBtnHeight+'px;" />');
    this.$closeBtn.bind('mouseup touchend', function(e) {
        self.hide();
    });

    this.$popUp.append(this.$closeBtn); */

    this.$overlay.appendTo(this.page.$container);
    this.$popUp.appendTo($(this.container));

    this.$overlay.css({
        'background' : this.page.ui.overlay.background,
        'opacity' : this.page.ui.overlay.opacity
    });

    this.$overlay.bind('mousedown touchstart', function(e) {
        self.hide();
    });

    this.$popUp.css({
        'width' : popupWidth + 'px',
        'height' : popupHeight + 'px',
        'background-image' : 'url("'+this.page._path +'bg/bg_Popup-Inputs.png")',
        'background-size': popupWidth + 'px '+ popupHeight +'px',
        'background-repeat' : 'no-repeat',
        'border-radius' : '5px',
        'text-align' : 'center'
    });

    return this;
};

AmpInputPopup.prototype.show = function(e, zone, amp){
    var self = this;
    this._zone = zone;
    this._amp = amp;
    this.clickedElement = $(e.currentTarget);
    this.zoneName = $(this.clickedElement.parent().prev());

    var top = 0,
        left = 0,
        offset = $(e.currentTarget).offset(),
        width = e.currentTarget.offsetWidth,
        parent = $(e.target).parent(),
        inputWidth = $(parent).width(),
        left2 = inputWidth /6;

    this.$popUp.css({
        'display': 'block'
    });

    top = (parent[0].offsetTop + parent[0].offsetHeight);
    left = (this.container.width() - this.$popUp.width())/2;

    if(offset.left < (inputWidth/2)) { //last one to the left
        left = left2;
    } else if(offset.left > (this.page.sizes.width - inputWidth)) { //last one to the right
        left = (this.container.width() - this.$popUp.width() - left2);
    }

    this.$overlay.css({
      'display' : 'block'
    });

    this.$popUp.css({
        'top' : top + 'px',
        'left' : left + 'px'
    });

    this.clickedElement.css({
        'z-index' : '103',
        'position' : 'relative'
    });

    this.zoneName.css({
        'z-index' : '103'
    });


    for (var i = 0; i < this._amp.inputs.length; i++) {

        Utils.addSimplePushGraphic({
            'img': this._amp.inputs[i],
            'params': {'input': i, 'self': this},
            'padding': '25px 10px 25px 10px',
            'display': 'inline-block',
            'height': 39,
            'width': 50,
            'container': this.$popUp,
            callback: function (e, param) {
                param.self.clickedElement.attr('src', param.self._amp.inputs[param.input]);
                param.self.tile.setPowerStatus('on');

                if (param.self._amp.zones[param.self._zone].power == "off") {
                    param.self._amp.executeCmd('POWER_ON', param.self._zone + 1, true);
                }

                var cmd = 'INPUT' + (param.input + 1);
                param.self._amp.executeCmd(cmd, param.self._zone + 1, true);
                param.self.hide();
            }
        });
    }
};

AmpInputPopup.prototype.hide = function(){

    if(this.clickedElement) {
        this.clickedElement.css({
            'z-index': '0'
        });
    }

    if(this.zoneName) {
        this.zoneName.css({
            'z-index': '0'
        });
    }

    this.$popUp.css({
        'display' : 'none'
    });

    this.$overlay.remove();
    this.$popUp.remove();
    this.page.inputPopVisible = false;
};

/********************************************
 * MSliderAmp
 */

/**
 *
 * @param path
 * @constructor
 */
MSliderAmp = function(path, page)
{
    this.page = page;
    this._div = null;               //div holding all components of this element
    this._thumbDiv = null;
    this._leftSpacerDiv = null;
    this._rightSpacerDiv = null;
    this.hasBg = true;

    this.continuous = false;
    this.step = 1;
    this.updateCallback = null;
    this.endCallback = null;

    this._path = path;
    this._minValue = 0;
    this._maxValue = 100;
    this._vertical = false;

    this._left = 0;
    this._top = 0;
    this._width = 0;

    this._thumbWidth = 73 * this.page.sizes.scaleFactor;
    this._sliderHeight = 55 * this.page.sizes.scaleFactor;
    this.stopPoints = [];

    //this._dragging = false;

    this.indicator = true;
    this.useStretching = true;

    /**
     *
     * @param inverted
     */
    MSliderAmp.prototype.setVertical = function(inverted) {

        this._vertical = true;

        if (inverted != null)
            this._inverted = inverted;
    }

    /**
     *
     * @param inverted
     */
    MSliderAmp.prototype.hasBackground = function(bg) {

        this.hasBg = bg;
    }

    /**
     *
     * @param min
     * @param max
     */
   MSliderAmp.prototype.setRange = function(min, max) {

        if (min)
            this._minValue = parseInt(min);

        if (max)
            this._maxValue = parseInt(max);
    }

    /**
     * Remove the Slider element from the dom
     */
    MSliderAmp.prototype.remove = function(){
        $(this._div).remove();
    };

    /**
     *
     * @param page
     * @param left
     * @param top
     * @param width
     */
    MSliderAmp.prototype.addToPage = function(page, left, top, width) {

        this._left = parseInt(left);
        this._top = parseInt(top);
        this._width = parseInt(width);
        var height = this._sliderHeight; 	//height of slider image

        //create main div
        //(bg.png is curently not used by aioremote)
        this._div = $('<div style="position:absolute; left: '+left+'px; top: '+top+'px; width: '+width+'px; height: '+height+'px; overflow: hidden; z-index: 2"></div>');

        this._div.css("-webkit-backface-visibility", "hidden");

        if(this.hasBg)
            this._div.css({
                'background-image' : 'url('+this._path+'bg.png)'
            });

        //we might rotate the right one and use it as left one when no thumb is shown
        leftImg = "left.png";
        leftImgStyle = "";
        if (this.indicator == false)
        {
            //http://css-tricks.com/snippets/css/flip-an-image/
            leftImg = "right.png";
            leftImgStyle = " -moz-transform: scaleX(-1); -webkit-transform: scaleX(-1); transform: scaleX(-1); filter: FlipH; -ms-filter: \"FlipH\";";
        }

        //add left and right end
//        $('<img style="position:absolute; left: 2px; clip: rect(0px 5px '+height+'px 0px);'+leftImgStyle+'" src="'+path+leftImg+'" draggable="false" />')
//        .appendTo(this._div);
        $('<div style="position:absolute; left: 0px; top: 0px; width: 8px; height: '+height+'px; overflow:hidden;"><img style="position:absolute; left: 2px;'+leftImgStyle+'" src="'+this._path+leftImg+'" draggable="false" /></div>')
            .appendTo(this._div);
        $('<img style="position:absolute; left: '+(width-34)+'px; clip: rect(0px 32px '+height+'px 27px);" src="'+this._path+'right.png" draggable="false" />')
            .appendTo(this._div);

        //add spacers
        this._leftSpacerDiv = $('<div style="position:absolute; left: 8px; top: 0px; width: 80px; height: '+height+'px; overflow: hidden;"></div>');
        if (this.useStretching)
            $('<img style="position:absolute; left: -100px; width: 100%; height: '+height+'px;'+leftImgStyle+'" src="'+this._path+leftImg+'" draggable="false" />')
                .appendTo(this._leftSpacerDiv);
        this._leftSpacerDiv.appendTo(this._div);

        this._rightSpacerDiv = $('<div style="position:absolute; left: 0px; top: 0px; width: 80px; height: '+height+'px; overflow: hidden;"></div>');
        $('<img style="position:absolute; left: 0px; width: '+(width+200)+'px; height: '+height+'px;" src="'+this._path+'right.png" draggable="false" />')
            .appendTo(this._rightSpacerDiv);
        this._rightSpacerDiv.appendTo(this._div);

        //add thumb
        this._thumbDiv = $('<div style="position:absolute; left: 0px; top: 0px; width: '+this._thumbWidth+'px; height: '+height+'px; overflow: hidden;"></div>');
        var timg = $('<img style="width: '+this._thumbWidth+'px; position:absolute; left: 0px; top: 0px;" src="'+path+'thumb.png" draggable="false" />')
            .appendTo(this._thumbDiv);
        //timg.bind('load', function(e) {
        //	alert("loaded");
        //});
        //timg.appendTo(this._thumbDiv);

        if (this.indicator == false)
        //this._thumbDiv.hide();
            this._thumbDiv.css("width", "0px");

        this._thumbDiv.appendTo(this._div);

        //init them to 0 like thumb
        this.setState(0);
        //this._leftSpacerDiv.css("width", Math.floor(100+this._thumbWidth/2-5)+"px");
        //this._rightSpacerDiv.css("left", Math.floor(this._thumbWidth/2+2)+"px");
        //this._rightSpacerDiv.css("width", Math.ceil(this._width-(this._thumbWidth/2)-9)+"px");


        if (this._vertical)
        {
            deg = 90;
            if (this._inverted)
                deg = -90;

            //this._div.css("-webkit-transform", "rotate3d(0,0,1,"+deg+"deg)");
            this._div.css("-webkit-transform", "rotate("+deg+"deg)");
            this._div.css("-moz-transform", "rotate("+deg+"deg)");
            this._div.css("-ms-transform", "rotate("+deg+"deg)");

            this._div.css("left", (this._left-(this._width/2)+(height/2)-2)+"px");
            this._div.css("top", (this._top+(this._width/2)-(height/2)+3)+"px");
        }

        var self = this;

        var clicked = false;
        this._div.bind('mousedown touchstart', function (e) {
            e.preventDefault();

            if(e.originalEvent && e.originalEvent.touches && e.originalEvent.touches.length) {
                e = e.originalEvent.touches[0];
            } else if(e.originalEvent && e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
                e = e.originalEvent.changedTouches[0];
            }

            self._thumbDiv.css("width", self._thumbWidth+"px");

            x = ((e.pageX-self._div.offset().left)+self._left);
            y = ((e.pageY-self._div.offset().top)+self._top);

            p = self.updateSlider(x, y);
            clicked = true;

            self.page.activeElement = self;

            return false;
        });

        this._div.bind('mousemove touchmove', function (e) {

            if(!clicked)
                return;
            e.preventDefault();

            if(e.originalEvent && e.originalEvent.touches && e.originalEvent.touches.length) {
                e = e.originalEvent.touches[0];
            } else if(e.originalEvent && e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
                e = e.originalEvent.changedTouches[0];
            }

            self._thumbDiv.css("width", self._thumbWidth+"px");

            x = ((e.pageX-self._div.offset().left)+self._left);
            y = ((e.pageY-self._div.offset().top)+self._top);

            p = self.updateSlider(x, y); //(e.touches[0]);

            return false;
        });

        //$(document).on('mouseup touchend', function(e) {
        this._div.bind('mouseup touchend', function (e) {

            clicked = false;

            e.preventDefault();

            if(e.originalEvent && e.originalEvent.touches && e.originalEvent.touches.length) {
                e = e.originalEvent.touches[0];
            } else if(e.originalEvent && e.originalEvent.changedTouches && e.originalEvent.changedTouches.length) {
                e = e.originalEvent.changedTouches[0];
            }

            self._thumbDiv.css("width", self._thumbWidth+"px");

            x = ((e.pageX-self._div.offset().left)/1+self._left);
            y = ((e.pageY-self._div.offset().top)/1+self._top);

            p = self.updateSlider(x, y); //(e.touches[0]);
            self.stop(p);
            return false;
        });


        this._div.appendTo(page);
    }

    /**
     *
     */
    MSliderAmp.prototype.stop = function(state) {

        if (this.indicator == false)
        //this._thumbDiv.hide();
            this._thumbDiv.css("width", "0px");

        if (this.endCallback)
            this.endCallback(state);
    }

    /**
     *
     * @param x
     * @param y
     * @returns {number}
     */
    MSliderAmp.prototype.updateSlider = function(x, y) {

        var pos = x - this._left;
        var maxPos = this._width - this._thumbWidth;

        if (this._vertical)
        {
            pos = y - this._top - 3;

            if (this._inverted)
                pos = maxPos - pos + this._thumbWidth;
        }

        pos = pos - this._thumbWidth/2;
        if (pos < 0)
            pos = 0;
        if (pos > maxPos)
            pos = maxPos;

        pos = Math.round(((this._maxValue - this._minValue) / maxPos) * pos) + this._minValue;

        pos = Math.floor(pos/this.step) * this.step;

        this.setState(pos);

        this._lastPos = pos;

        if (this.updateCallback)
        {
            this.updateCallback(pos);
        }

        return pos;
    }

    /**
     *
     * @param state
     * @returns {*}
     */
    MSliderAmp.prototype.setState = function(state) {

        var maxPos = this._width - this._thumbWidth;

        var pos =  Math.round((state - this._minValue) / ((this._maxValue - this._minValue) / maxPos));
        if (pos < 0)
            pos = 0;
        if (pos > maxPos)
            pos = maxPos;

        this._thumbDiv.css("left", pos+"px");

        this._leftSpacerDiv.css("width", Math.floor(pos+100+this._thumbWidth/2-5)+"px");
        this._rightSpacerDiv.css("left", Math.floor(pos+this._thumbWidth/2+2)+"px");
        this._rightSpacerDiv.css("width", Math.ceil(this._width-(pos+this._thumbWidth/2)-9)+"px");

        return state;
    }

    MSliderAmp.prototype.hide = function() {
        this._div.css({display:'none'})
    }

    MSliderAmp.prototype.show = function() {
        this._div.css({display:'block'})
    }

};

/**
 * Simple Text
 * @param text
 * @param upperCase
 * @constructor
 */
Mtext = function(text, upperCase){
    this.upperCase = upperCase;

    if(this.upperCase)
        this.text = text.toUpperCase();
    else
        this.text = text;
};

/**
 * Sets the Text
 * @param text
 */
Mtext.prototype.setText = function(text) {

    if(this.upperCase)
        this.div.html(text.toUpperCase());
    else
        this.div.html(text);
};

/**
 * Adds Text to page
 * @param page
 * @param top
 * @param left
 * @param width
 * @param height
 */
Mtext.prototype.addToPage = function(page, top, left, width, height ){
    this.div = $('<div style="text-align:center; position:absolute; color: white; font-size:20px; top:'+top+'px; left:'+left+'px; width:'+width+'px; height:'+height+'px;">'+this.text+'</div>');
    //page.append(this.div);
    this.div.appendTo(page);
};

/**
 *
 * @param object
 */
Mtext.prototype.css = function(object) {
   this.div.css(object);
};

var Utils = (function(){

    var module = {};

    /**
     * Adds a simple push graphic to a container
     * @param options
     * @returns {*|$|HTMLElement}
     */
    module.addSimplePushGraphic = function(options) {

        var defaults = {
            'img' : '',
            'padding' : '0px 0px 0px 0px',
            'margin' : '0 0 0 0',
            'zindex' : '0'
        };

        if(options) {
            $.extend(true, defaults,options);
        }

        var $btndiv = $('<div style="z-index : '+defaults.zindex+'; padding: '+defaults.padding+'; height:'+defaults.height+'px; width:'+defaults.width+'px; display:inline; color: white; text-align: center; overflow: hidden; white-space: nowrap; text-overflow: ellipsis;"><img class="button" style="margin:'+defaults.margin+'; height:'+defaults.height+'px; width:'+defaults.width+'px" src="'+ defaults.img+'" /></div>');

        if(defaults.display) {
            $btndiv.css({
                'display' : defaults.display
            });
        }

        if(defaults.top && defaults.left) {
            $btndiv.css({
                'position': 'absolute',
                'top': defaults.top + 'px',
                'left': defaults.left + 'px'
            });
        }

        if(defaults.top && defaults.right) {
            $btndiv.css({
                'position': 'absolute',
                'top': defaults.top + 'px',
                'right': defaults.right + 'px'
            });
        }

        $btndiv.bind('mousedown touchstart', function (e) {
            //cancelled = false;
            if(defaults.callback) {
                if (defaults.params) {
                    defaults.callback(e, defaults.params);
                } else {
                    defaults.callback(e);
                }
            }

        });

        //defaults.container.append($btndiv);
        $btndiv.appendTo(defaults.container);

        return $btndiv;
    };

    return module;

})();

var MCAAM = function(page) {
    var requests = [],
        runCallback;

    return {
        addReq:  function(opt) {
            requests.push(opt);
        },
        /*removeReq:  function(opt) {
            if( $.inArray(opt, requests) > -1 )
                requests.splice($.inArray(opt, requests), 1);
        },*/
        setRunCallback : function(callback){
            runCallback = callback;
        },
        run: function() {

            var self = this,
                oriSuc;

            if( requests.length ) {
                oriSuc = requests[0].complete;

                requests[0].complete = function() {
                    if( typeof(oriSuc) === 'function' ) oriSuc();
                    requests.shift();
                    self.run.apply(self, []);
                };

                $.ajax(requests[0]);
            } else {
                setTimeout(function(){
                    page.hideSpinner();

                    if(runCallback)
                        runCallback();
                }, 500);
            }
        },
        requests : requests
    };
};