/**
 * Media Player.
 * AIO REMOTE NEO only.
 */
var MMediaPlayer = function() {


    this._div = null; // div holding all components of this element
    this._height = 0;
    this._layout = null;
    this._left = 0;
    this._top = 0;
    this._width = 0;

    this._cover = null;
    this._meta = null;
    this._metaArtist = null;
    this._metaTitle = null;
    this._metaZone = null;

    this._browseHistory = [];
    this._browsePending = false;
    this._currentBrowsingData = [];
    this._lastDir = null;
    this._loadPlayerPending = false;
    this._timeoutUpdate = 0;
    this._updateInterval = 1000; // [ms]

    this.coverDefault = './images/cover.png';
    this.cssClass = null; // aioremote2
    this.device = null;
    this.entriesPerPage = 30;
    this.mediaServer = null;
    this.pageIndex = 0;
    this.player = null;
    this.running = false;


    /**
     * Build a directory list item.
     * @param  {Object} entry Data.
     * @return {Object}       List item (jQuery).
     */
    MMediaPlayer.prototype._buildDirectoryItem = function(entry) {
        var $item = $('<li class="dir"></li>');
        var $icon = $('<div class="icon"></div>');
        var $text = $('<span class="text"></span>');

        if (this.device.sys == 'sonos' && entry.id == 'A:') {
            $text.text('Music');
        }
        else {
            $text.text(entry.title);
        }

        if (this._listItemAddIcon(entry, $icon)) {
            $item.addClass('hasArt');
        }

        $item.append($icon);
        $item.append($text);
        this._listItemAddPlay(entry, $item);

        $item.on(XC.END_EVT, this._handleListSelect.bind(this));

        return $item;
    };


    /**
     * Build a file list item.
     * @param  {Object} entry Data.
     * @param  {Number} i     List index.
     * @return {Object}       List item (jQuery).
     */
    MMediaPlayer.prototype._buildFileItem = function(entry, i) {
        var $item = $('<li class="file"></li>');
        var $icon = $('<div class="icon"></div>');
        var $text = $('<span class="text"></span>');

        $text.text(entry.title);

        if (this._listItemAddIcon(entry, $icon)) {
            $item.addClass('hasArt');
        }

        $item.append($icon);
        $item.append($text);
        this._listItemAddPlay(entry, $item);

        if (typeof i === 'number') {
            $item.attr('data-index', i);
        }

        $item.on(XC.END_EVT, this._handleListSelect.bind(this));

        return $item;
    };


    /**
     * Build the header/controls area.
     * @return {Object} (jQuery.)
     */
    MMediaPlayer.prototype._buildHeader = function() {
        this._cover = $('<div class="cover"></div>');
        this._cover.css({
            backgroundImage: 'url("' + this.coverDefault + '")',
            height: this._height,
            width: this._height
        });
        this._cover.css(this._layout.css.cover);

        this._cover.on('error', function(ev) {
            XC.debugf('[MMediaPlayer._buildHeader] Failed to load cover: ' + ev.target.src, 'error');
        });

        this._meta = $('<div class="meta"></div>');
        this._meta.css({
            width: this._width - this._height - 20
        });

        this._metaTitle = $('<span class="title none">' + XC_Text['mp_noTitleSelected'] + '</span>');
        this._metaArtist = $('<span class="artist"></span>');
        this._metaZone = $('<span class="zone"></span>');

        this._meta.append(this._metaZone);
        this._meta.append(this._metaTitle);
        this._meta.append(this._metaArtist);

        var $header = $('<div class="header"></div>');
        $header.append(this._cover);
        $header.append(this._meta);
        $header.on(XC.END_EVT, this.toggleList.bind(this));

        $header.css({
            height: this._height
        });
        $header.css(this._layout.css.header);

        return $header;
    };


    /**
     * Build the list container.
     * @return {Object} (jQuery.)
     */
    MMediaPlayer.prototype._buildList = function() {
        var $lc = $('<div class="listcontainer"></div>');
        $lc.css({
            top: this._height,
            width: this._width
        });
        $lc.css(this._layout.css.list);

        var $tabs = $('<div class="tabs"></div>');
        $lc.append($tabs);

        var $browse = $('<div class="tab first active" id="mmp-tab-browse"></div>');
        $browse.text(XC_Text['mp_tabBrowse']);
        $browse.on(XC.END_EVT, this._selectTab.bind(this));
        $tabs.append($browse);

        var $zones = $('<div class="tab" id="mmp-tab-zones"></div>');
        $zones.text(XC_Text['mp_tabZones']);
        $zones.on(XC.END_EVT, this._selectTab.bind(this));
        $tabs.append($zones);

        var $back = $('<div class="back"></div>');
        $back.on(XC.END_EVT, this.browseBack.bind(this));

        var $nav = $('<div class="nav"></div>');
        $nav.css(this._layout.css.side);
        $nav.append($back);

        var $list = $('<ul class="list"></ul>');
        $list.css({
            maxHeight: 300,
            minHeight: 60
        });

        $nav.append($list);
        $lc.append($nav);

        return $lc;
    };


    /**
     * Build a page navigation list item.
     * @return {Object} List item (jQuery).
     */
    MMediaPlayer.prototype._buildPageNavItem = function() {
        var dataNum = this._currentBrowsingData.length;
        var hasPrev = (this.pageIndex > 0);
        var hasNext = (dataNum > 0 && dataNum == this.entriesPerPage);

        if (!hasPrev && !hasNext) {
            return null;
        }

        var $pageNav = $('<li class="page-nav"></li>');
        var $prev = $('<div class="prev"></div>');
        var $next = $('<div class="next"></div>');

        if (hasPrev) {
            $prev.text(XC_Text['mp_prevPage'] || '_< zurück');
            $prev.on(XC.END_EVT, this.showPrevPage.bind(this));
        }

        if (hasNext) {
            $next.text(XC_Text['mp_nextPage'] || '_weiter >');
            $next.on(XC.END_EVT, this.showNextPage.bind(this));
        }

        $pageNav.append($prev);
        $pageNav.append($next);

        return $pageNav;
    };


    /**
     * Build a room list item.
     * @param  {Object} r  Room data.
     * @param  {Object} zp Zone player.
     * @return {Object}    List item (jQuery).
     */
    MMediaPlayer.prototype._buildRoomItem = function(r, zp) {
        var $status = $('<span class="status"></span>');
        var $text = $('<span class="text"></span>');
        $text.text(r.name);

        var $item = $('<li></li>');
        $item.data('room', JSON.stringify(r));
        $item.on(XC.END_EVT, this._toggleRoomZoneAssign.bind(this));

        if (zp.rooms[r.uuid]) {
            $item.addClass('in-zone');
            $status.text('\u2714'); // check mark
        }
        else {
            $status.text('\u2718'); // cross/x mark
        }

        $item.append($status);
        $item.append($text);

        return $item;
    };


    /**
     * Build a zone list item.
     * @param  {Object} z Zone data.
     * @return {Object}   List item (jQuery).
     */
    MMediaPlayer.prototype._buildZoneItem = function(z) {
        var rooms = [];

        for (var key in z.rooms) {
            var r = z.rooms[key];
            rooms.push(r);
        }

        var name = rooms.join(', ');

        if (name.length === 0) {
            name = z.uuid;
        }

        var $text = $('<span class="text"></span>');
        $text.text(name);

        var $play = $('<div class="play"></div>');
        $play.on(XC.END_EVT, this._selectZone.bind(this));

        var $item = $('<li class="playable"></li>');
        $item.attr('id', 'zone-' + z.uuid);
        $item.append($text);
        $item.append($play);

        if (this.device.sys == 'sonos') {
            $item.on(XC.END_EVT, this._selectZone.bind(this));
        }
        else {
            $item.on(XC.END_EVT, this._handleZoneSelect.bind(this));
        }

        return $item;
    };


    /**
     * Check if the given data object contains play information.
     * @param  {Object}  data List item data.
     * @return {Boolean}      True if playable, false otherwise.
     */
    MMediaPlayer.prototype._canPlay = function(data) {
        if (!data) {
            return false;
        }

        if (data.res) {
            return true;
        }

        return (
            this.device.sys == 'raumfeld' &&
            data.type == 'item' &&
            typeof data.parentID === 'string' &&
            data.parentID.indexOf('0/RadioTime/LocalRadio') === 0
        );
    };


    /**
     * Get a filter list of which directories not to display.
     * @return {Array<String>}
     */
    MMediaPlayer.prototype._getDirectoryFilter = function() {
        var dirFilter = [];

        if (this.device.sys == 'raumfed') {
            dirFilter = ['0/Spotify', '0/Zones', '0/Renderers'];
        }
        else if (this.device.sys == 'sonos') {
            dirFilter = ['R:'];
        }

        return dirFilter;
    },


    /**
     * Handler for list selection.
     * @param {Event} ev
     */
    MMediaPlayer.prototype._handleListSelect = function(ev) {
        if (RemoteController && RemoteController.mouseMoved()) {
            return;
        }

        var target = ev.currentTarget || ev.target;

        if (this.device.sys == 'sonos' && target.className == 'clear-queue' && target.id.length > 0) {
            var queueID = target.id;
            this.clearQueue(queueID);

            return;
        }

        var entryID = Number(target.id.substr(6));
        var data = this._currentBrowsingData[entryID];

        if (!data) {
            return;
        }

        var addToQueue = (ev.target.className == 'add-to-queue');

        // If play button in the list item has been hit
        // or it can be played but not browsed, then select
        // the file for playback.
        if (ev.target.className == 'play' || addToQueue || data.type == 'item') {
            if (this._canPlay(data)) {
                var index = Number(target.getAttribute('data-index'));
                this.selectFile(data, index, addToQueue);
            }
        }
        else {
            // Browse into the directory.
            if (data.type == 'container') {
                this.pageIndex = 0;

                this.browse(data.id, 0, function(err, res) {
                    if (err) {
                        XC.debugf(err, 'error');
                        return;
                    }

                    this._lastDir = data.id;
                    this._browseHistory.push(data.parentID);
                    this.updateList(res);
                }.bind(this));
            }
        }
    };


    /**
     * Handler for zone list selection.
     * @param {Event} ev
     */
    MMediaPlayer.prototype._handleZoneSelect = function(ev) {
        if (RemoteController && RemoteController.mouseMoved()) {
            return;
        }

        var target = ev.currentTarget || ev.target;

        if (ev.target.className == 'play') {
            return;
        }

        var uuid = target.id.substr(5);

        this.buildRoomsList();
    };


    /**
     * Build a list of all rooms for a zone.
     */
    MMediaPlayer.prototype.buildRoomsList = function() {
        var $back = this._listContainer.find('.back');
        $back.text('<');

        var $list = this._listContainer.find('.list');
        $list.html('');

        var self = this;

        this.player.getCurrentZone(function(err, zp) {
            if (err) {
                XC.debugf('[MMediaPlayer.buildRoomsList]' +
                    ' ERROR (getCurrentZone): ' + (err.message || err), 'error');
                return;
            }

            self.player.getAllRoomInfo(function(err, rooms) {
                if (err) {
                    XC.debugf('[MMediaPlayer.buildRoomsList]' +
                        ' ERROR (getAllRoomInfo): ' + (err.message || err), 'error');
                    return;
                }

                var $first = $('<li class="title"></li>');
                $first.text((XC_Text['mp_rooms'] || '_Räume') + ':');
                $list.append($first);

                for (var i = 0; i < rooms.length; i++) {
                    var r = rooms[i];
                    var $item = self._buildRoomItem(r, zp);

                    $list.append($item);
                }
            });
        });
    };


    /**
     * Maybe add some icon to the list item, like cover art.
     * @param  {Object}  entry List item data.
     * @param  {Object}  $icon Icon container (jQuery).
     * @return {Boolean}       True if an icon has been added, false otherwise.
     */
    MMediaPlayer.prototype._listItemAddIcon = function(entry, $icon) {
        if (!entry || !entry.info || typeof entry.info.albumArtURI !== 'string') {
            return false;
        }

        $icon.css({
            backgroundImage: 'url("' + entry.info.albumArtURI + '")',
            display: 'block'
        });

        return true;
    };


    /**
     * Add a play button if item/dir can be played.
     * @param {Object} entry List item data.
     * @param {Object} $item List item (jQuery).
     */
    MMediaPlayer.prototype._listItemAddPlay = function(entry, $item) {
        if (!this._canPlay(entry)) {
            return;
        }

        if (this.device.sys == 'sonos' && entry.parentID != 'Q:') {
            var $add = $('<div class="add-to-queue"></div>');
            $item.addClass('queueable');
            $item.append($add);
        }

        var $play = $('<div class="play"></div>');
        $item.addClass('playable');
        $item.append($play);
    };


    /**
     * Callback for the loaded device.
     * @param {Error|null} err    Error.
     * @param {Object}     device Loaded device.
     */
    MMediaPlayer.prototype._onDeviceLoad = function(err, device) {
        if (err) {
            XC.debugf('[MMediaPlayer.addToPage] ' + (err.message || err), 'error');
            this.showMessage('error', XC_Text['mp_errDevice']);

            return;
        }

        this.device = device;

        // Zones are only available for the "Raumfeld" system
        // and if the master device is assigned.
        var hasZones = ['raumfeld', 'sonos'];
        if (hasZones.indexOf(this.device.sys) < 0 || this.device.type != 'master') {
            $('#mmp-tab-zones').remove();
        }

        var okay = this.loadPlayer();

        if (!okay) {
            return;
        }

        this.loadMediaServer();
    };


    /**
     * Callback for the loaded media server list.
     * @param {Error|null}                       err Error.
     * @param {Array<x.upnp.MediaServer.Server>} res List of media servers.
     */
    MMediaPlayer.prototype._onMediaServerLoad = function(err, res) {
        if (err) {
            XC.debugf('[MMediaPlayer.loadMediaServer] ' + (err.message || err), 'error');
            this.showMessage('error', XC_Text['mp_errLoadMediaServer']);

            return;
        }
        if (!res || res.length < 1) {
            XC.debugf('[MMediaPlayer.loadMediaServer]' +
                ' No error, but also no result.', 'error');
            this.showMessage('info', XC_Text['mp_noMediaServerFound']);

            return;
        }

        for (var i = 0; i < res.length; i++) {
            var ms = res[i];

            if (this.device.sys == 'raumfeld' && ms._raumfeldMaster) {
                this.mediaServer = ms;
                break;
            }
            else if (this.device.sys == 'sonos' && ms._sonosServer) {
                this.mediaServer = ms;
                break;
            }
            // Bose has no media server.
        }

        // Fallback: Just use the first one.
        if (!this.mediaServer) {
            this.mediaServer = res[0];
        }

        this._browseHistory = [];

        if (this.player) {
            this.startUpdateLoop();
            this.hideMessage();
        }
    };


    /**
     * Callback for the loaded player.
     * @param {Error|null} err Error.
     * @param {Object}     p   Loaded player.
     */
    MMediaPlayer.prototype._onPlayerLoad = function(err, p) {
        this._loadPlayerPending = false;

        if (err) {
            XC.debugf('[MMediaPlayer.loadPlayer] ' + (err.message || err), 'error');
            this.showMessage('error', XC_Text['mp_errLoadPlayer']);

            return;
        }
        else if (!p) {
            XC.debugf('[MMediaPlayer.loadPlayer] No error, but could not load player.', 'error');
            this.showMessage('error', XC_Text['mp_errLoadPlayer']);

            return;
        }

        this.player = p;

        if (this.mediaServer) {
            this.startUpdateLoop();
            this.hideMessage();
        }
    };


    /**
     * Handle tab selection.
     * @param {Event} ev
     */
    MMediaPlayer.prototype._selectTab = function(ev) {
        var $tab = $(ev.delegateTarget || ev.target);
        $tab.parent().children('.tab').removeClass('active');
        $tab.addClass('active');

        var id = $tab.attr('id');

        switch (id) {
            case 'mmp-tab-browse':
                this.browseLastOpenedDir();
                break;

            case 'mmp-tab-zones':
                this.listZones();
                break;
        }
    };


    /**
     * Select the zone.
     * @param {Event} ev
     */
    MMediaPlayer.prototype._selectZone = function(ev) {
        ev.preventDefault();

        var item = ev.target;

        if (!item.id) {
            item = item.parentNode;
        }

        var zone = item.id.substr(5);

        if (!this.player) {
            return;
        }

        this.player.setCurrentZone(zone, function(err, zp) {
            if (err) {
                XC.debugf('[MMediaPlayer._selectZone]' +
                    ' ERROR (setCurrentZone): ' + err.message + ' (zone: "' + zone + '")', 'error');
                return;
            }

            XC.debugf('[MMediaPlayer._selectZone] Set zone to: ' + zone);
        }.bind(this));
    };


    /**
     * Add a room to a zone or remove it.
     * @param {Event} ev
     */
    MMediaPlayer.prototype._toggleRoomZoneAssign = function(ev) {
        var $item = $(ev.delegateTarget || ev.target);
        var r = $item.data('room');

        if (!r) {
            XC.debugf('[MMediaPlayer._toggleRoomZoneAssign]' +
                ' ERROR: No room data in list item.', 'error');
            return;
        }

        r = JSON.parse(r);

        this.player.getCurrentZone(function(err, zp) {
            if (err) {
                XC.debugf('[MMediaPlayer._toggleRoomZoneAssign]' +
                    ' ERROR (getCurrentZone): ' + err.message, 'error');
                return;
            }

            var numRooms = 0;

            for (var key in zp.rooms) {
                numRooms++;
            }

            // Last room in the zone. Do not remove.
            if (numRooms === 1) {
                return;
            }

            // If room in zone, remove from zone.
            if (zp.rooms[r.uuid]) {
                delete zp.rooms[r.uuid];

                this.player.dropRoom(r.uuid, function(err) {
                    if (err) {
                        XC.debugf('[MMediaPlayer._toggleRoomZoneAssign]' +
                            ' ERROR (dropRoom): ' + err.message, 'error');
                    }
                });

                $item.removeClass('in-zone');
                $item.children('.status').text('\u2718');
            }
            // Otherwise add room to zone.
            else {
                zp.addRoom(r.uuid, r.name);
                $item.addClass('in-zone');
                $item.children('.status').text('\u2714');
            }
        }.bind(this));
    };


    /**
     * Update loop.
     * @see MMediaPlayer.prototype.startUpdateLoop()
     * @see MMediaPlayer.prototype.stopUpdateLoop()
     */
    MMediaPlayer.prototype._updateLoop = function() {
        var self = this;

        if (!this.running || !this.player || !this.mediaServer) {
            return;
        }

        this.getPlayInfo(function(err, data) {
            var wait = 0;

            if (err) {
                XC.debugf('[MMediaPlayer._updateLoop]' +
                    ' ERROR (getPlayInfo): ' + err.message, 'error');
                wait = 4000;
            }

            if (data) {
                self.updateHeader(data);
            }

            self._timeoutUpdate = setTimeout(function() {
                self._updateLoop();
            }, self._updateInterval + wait);
        });
    };


    /**
     * Add the element to the page.
     * @param {MPage}  page
     * @param {Number} left
     * @param {Number} top
     * @param {Number} width
     * @param {Number} height
     */
    MMediaPlayer.prototype.addToPage = function(page, left, top, width, height) {
        APP.injectCSS('aio2-mmp', MMediaPlayer.CSS);

        this._left = parseInt(left, 10);
        this._top = parseInt(top, 10);
        this._width = parseInt(width, 10);
        this._height = parseInt(height, 10);

        this._div = $('<div></div>');
        var css = {
            height: this._height,
            left: this._left,
            position: 'absolute',
            top: this._top,
            width: this._width,
            zIndex: 2
        };

        if (this.cssClass) {
            this._div.addClass(this.cssClass);
            delete css.position;
        }

        this._div.css(css);

        this._header = this._buildHeader();
        this._listContainer = this._buildList();

        this._message = $('<div class="msg"></div>');
        this._message.css({
            height: this._height - 40,
            width: this._width - 40
        });

        this._div.append(this._header);
        this._div.append(this._listContainer);
        this._div.append(this._message);

        this._div.appendTo(page._contentDiv);

        page.on('REMOTE_PAUSE', function(ev) {
            this.stopUpdateLoop();
        }.bind(this));

        page.on('REMOTE_RESUME', function(ev) {
            this.startUpdateLoop();
        }.bind(this));

        this.showMessage('info', XC_Text['mp_loading'] || '_Lädt …');

        CommandHandler.getDeviceByIndex(
            this.deviceIndex,
            this._onDeviceLoad.bind(this)
        );
    };


    /**
     * Browse the source list.
     * @param {String}   path
     * @param {Number}   pageIndex
     * @param {Function} cb        Callback.
     */
    MMediaPlayer.prototype.browse = function(path, pageIndex, cb) {
        if (this.device.sys == 'bose') {
            return;
        }

        if (!this.mediaServer) {
            XC.debugf('[MMediaPlayer.browse] Media server not set.', 'warn');
            return;
        }

        path = path || '0';
        var start = pageIndex * this.entriesPerPage;
        var howMany = this.entriesPerPage;

        this.player.browser(this.mediaServer, path, start, howMany, cb);
    };


    /**
     * Go back one hierarchy level.
     * @param {Event} ev
     */
    MMediaPlayer.prototype.browseBack = function(ev) {
        var $tabZones = $('#mmp-tab-zones');

        if ($tabZones.hasClass('active')) {
            this.listZones();
            return;
        }

        if (this._browseHistory.length < 1) {
            return;
        }

        if (this._browsePending) {
            return;
        }

        var bh = this._browseHistory;
        var goto = bh.splice(bh.length - 1, 1)[0];

        this._browsePending = true;

        this.browse(goto, 0, function(err, res) {
            this._browsePending = false;

            if (err) {
                XC.debugf('[MMediaPlayer.browseBack] ' + err.message, 'error');
                return;
            }

            this._lastDir = goto;
            this.updateList(res);
        }.bind(this));
    };


    /**
     * Browse the last opened directory.
     */
    MMediaPlayer.prototype.browseLastOpenedDir = function() {
        this._browsePending = true;

        this.browse(this._lastDir, this.pageIndex, function(err, res) {
            this._browsePending = false;

            if (err) {
                XC.debugf('[MMediaPlayer.browseLastOpenDir] ' + err.message, 'error');
                return;
            }

            this.updateList(res);
        }.bind(this));
    };


    /**
     * Clear a queue (Sonos).
     * @param {String}   queueID Queue ID.
     * @param {Function} cb      Callback.
     */
    MMediaPlayer.prototype.clearQueue = function(queueID, cb) {
        if (!this.player) {
            XC.debugf('[MMediaPlayer.clearQueue] Player not loaded.', 'error');
            return;
        }

        this.player.clearQueue(queueID, function(err) {
            if (err) {
                XC.debugf('[MMediaPlayer.clearQueue] ' + err.message, 'error');
            }
        });
    };


    /**
     * Get the node module for the given device system.
     * @param  {String} sys The system.
     * @return {Object}     The module.
     */
    MMediaPlayer.prototype.getModule = function(sys) {
        var module = null;

        switch (sys) {
            case 'raumfeld':
                module = require('xnm.aio.raumfeld.js');
                break;

            case 'bose':
                module = require('xnm.aio.bose.js');
                break;

            case 'sonos':
                module = require('xnm.aio.sonos.js');
                break;

            default:
                XC.debugf('[MMediaPlayer.getModule]' +
                    ' Unknown sys: "' + sys + '"', 'error');
        }

        return module;
    };


    /**
     * Request the current play info.
     * @param {Function} cb Callback.
     */
    MMediaPlayer.prototype.getPlayInfo = function(cb) {
        if (!this.player) {
            return;
        }

        if (typeof this.player.getCurrentZone != 'function') {
            var err = new Error('No function "getCurrentZone" in player.');
            XC.debugf('[MMediaPlayer.getPlayInfo] ' + err.message, 'error');

            // Unset player, stopping the update loop.
            this.player = null;
            cb && cb(err, null);

            return;
        }

        var self = this;

        this.player.getCurrentZone(function(err, zonePlayer) {
            if (err) {
                XC.debugf('[MMediaPlayer.getPlayInfo]' +
                    ' ERROR (getCurrentZone): ' + err.message, 'error');
                cb && cb(err, null);

                return;
            }

            if (!zonePlayer) {
                err = new Error('No zonePlayer in callback.');
                XC.debugf('[MMediaPlayer.getPlayInfo]' +
                    ' ERROR (getCurrentZone): ' + err.message, 'error');
                cb && cb(err, null);

                return;
            }

            // Bose
            if (zonePlayer.masterPlayer) {
                zonePlayer = zonePlayer.masterPlayer;
            }
            // Sonos
            else if (self.device.sys == 'sonos' && zonePlayer.player) {
                zonePlayer = zonePlayer.player;
            }

            if (typeof zonePlayer.getPlayInfo != 'function') {
                err = new Error('No function "getPlayInfo" in zonePlayer.');
                XC.debugf('[MMediaPlayer.getPlayInfo]' +
                    ' ERROR (getCurrentZone): ' + err.message, 'error');
                cb && cb(err, null);

                return;
            }

            zonePlayer.getPlayInfo(function(err, info) {
                if (err) {
                    XC.debugf('[MMediaPlayer.getPlayInfo]' +
                        ' ERROR (getPlayInfo): ' + err.message, 'error');
                    cb && cb(err, null);

                    return;
                }

                var data = {
                    info: info,
                    zone: null
                };

                if (self.device.sys == 'bose') {
                    zonePlayer.getRooms(function(err, rooms) {
                        if (err) {
                            XC.debugf('[MMediaPlayer.getPlayInfo]' +
                                ' ERROR (getRooms): ' + err.message, 'error');
                            cb && cb(err, null);

                            return;
                        }

                        data.zone = rooms;
                        cb && cb(null, data);
                    });
                }
                else if (self.device.sys == 'sonos') {
                    data.zone = [zonePlayer.zoneName];
                    cb && cb(null, data);
                }
                else {
                    data.zone = zonePlayer.getRooms();
                    cb && cb(null, data);
                }
            });
        });
    };


    /**
     * Hide the message.
     */
    MMediaPlayer.prototype.hideMessage = function() {
        this._message.text('');
        this._message.hide();
    };


    /**
     * List the available zones.
     */
    MMediaPlayer.prototype.listZones = function() {
        if (!this.player) {
            XC.debugf('[MMediaPlayer.listZones] Player not set.', 'warn');
            return;
        }

        var buildList = function(zones) {
            var $list = this._listContainer.find('.list');
            $list.html('');

            var $first = $('<li class="title"></li>');
            $first.text((XC_Text['mp_zones'] || '_Zonen') + ':');
            $list.append($first);

            for (var i = 0; i < zones.length; i++) {
                var z = zones[i];
                var $item = this._buildZoneItem(z);

                $list.append($item);
            }
        }.bind(this);

        if (this.device.sys == 'sonos') {
            this.player.getZones(function(err, zones) {
                if (err) {
                    XC.debugf('[MMediaPlayer.listZones]' +
                        ' ERROR (getZones): ' + err.message, 'error');
                    return;
                }

                var zonesArr = [];

                for (var key in zones) {
                    var z = zones[key];
                    var rooms = [];

                    if (Array.isArray(z.members)) {
                        for (var i = 0; i < z.members.length; i++) {
                            var member = z.members[i];

                            if (member && member.name) {
                                rooms.push(member.name);
                            }
                        }
                    }

                    zonesArr.push({
                        rooms: rooms,
                        uuid: zones[key].id
                    });
                }

                buildList(zonesArr);
            }.bind(this));
        }
        else {
            this.player.getAllZoneInfo(function(err, zones) {
                if (err) {
                    XC.debugf('[MMediaPlayer.listZones]' +
                        ' ERROR (getAllZoneInfo): ' + err.message, 'error');
                    return;
                }

                buildList(zones);
            }.bind(this));
        }
    };


    /**
     * Load the media server.
     * @return {Boolean} True if loading can be attempted, false otherwise.
     */
    MMediaPlayer.prototype.loadMediaServer = function() {
        // Bose has no media server.
        if (this.device.sys == 'bose') {
            this.mediaServer = true;

            if (this.player) {
                this.startUpdateLoop();
                this.hideMessage();
            }

            return true;
        }

        var msModule = this.getModule(this.device.sys);

        if (!msModule) {
            return false;
        }

        msModule.getMediaServer(this._onMediaServerLoad.bind(this));

        return true;
    };


    /**
     * Load the player.
     * @return {Boolean} True if loading can be attempted, false if missing information.
     */
    MMediaPlayer.prototype.loadPlayer = function() {
        if (!this.device) {
            XC.debugf('[MMediaPlayer.loadPlayer] No device set.', 'error');
            return false;
        }

        var playerModule = this.getModule(this.device.sys);

        if (!playerModule) {
            return false;
        }

        if (this._loadPlayerPending) {
            XC.debugf('[MMediaPlayer.loadPlayer]' +
                ' Still waiting for previous request.', 'warn');

            return true;
        }

        this._loadPlayerPending = true;

        // On first load the player is returned in the callback. On later
        // requests it is returned directly from cache and the callback not
        // used.
        var pDirect = playerModule.getPlayer(
            this.device,
            this._onPlayerLoad.bind(this)
        );

        if (pDirect) {
            this.player = pDirect;

            if (this.mediaServer) {
                this.startUpdateLoop();
                this.hideMessage();
            }
        }

        return true;
    },


    /**
     * Play a media resource.
     * @param {Object}  data
     * @param {Number}  index
     * @param {Boolean} addToQueue
     */
    MMediaPlayer.prototype.play = function(data, index, addToQueue) {
        if (!this.player || !this.mediaServer) {
            XC.debugf('[MMediaPlayer.play] Player or media server not set.', 'warn');
            return;
        }

        if (this.device.sys == 'sonos') {
            if (addToQueue) {
                this.player.addToQueue(this.mediaServer, data, function(err) {
                    if (err) {
                        XC.debugf('[MMediaPlayer.play]' +
                            ' ERROR (addToQueue): ' + err.message, 'error');
                    }
                });
            }
            else {
                this.player.addToQueueAndPlay(this.mediaServer, data, function(err) {
                    if (err) {
                        XC.debugf('[MMediaPlayer.play]' +
                            ' ERROR (addToQueueAndPlay): ' + err.message, 'error');
                    }
                });
            }
        }
        else {
            if (typeof index === 'number' && !isNaN(index) && data.res) {
                this.player.playFromContainer(this.mediaServer, data, index, function(err) {
                    if (err) {
                        XC.debugf('[MMediaPlayer.play]' +
                            ' ERROR (playFromContainer): ' + err.message, 'error');
                    }
                });
            }
            else {
                this.player.play(this.mediaServer, data, function(err) {
                    if (err) {
                        XC.debugf('[MMediaPlayer.play]' +
                            ' ERROR (play): ' + err.message, 'error');
                    }
                });
            }
        }

        // Make sure the update loop is running.
        this.startUpdateLoop();
    };


    /**
     * Select a file for playblack.
     * @param {Object}  data       Item data.
     * @param {Number}  index      List index of the item.
     * @param {Boolean} addToQueue True to add to current queue instead of playing.
     */
    MMediaPlayer.prototype.selectFile = function(data, index, addToQueue) {
        this.updateHeader(data);
        this.play(data, index, addToQueue);
    };


    /**
     * Show a message overlay.
     * @param {String} type Type of the message ("info", "error").
     * @param {String} msg  Message to display.
     */
    MMediaPlayer.prototype.showMessage = function(type, msg) {
        this._message.removeClass('info error');
        this._message.addClass(type);
        this._message.text(msg);
        this._message.show();
    };


    /**
     * Show the next page of the currently browsed directory.
     * @param {Event} ev
     */
    MMediaPlayer.prototype.showNextPage = function(ev) {
        this.pageIndex++;

        this.browse(this._lastDir, this.pageIndex, function(err, res) {
            if (err) {
                XC.debugf(err, 'error');
                return;
            }

            this.updateList(res);
        }.bind(this));
    };


    /**
     * Show the previous page of the currently browsed directory.
     * @param {Event} ev
     */
    MMediaPlayer.prototype.showPrevPage = function(ev) {
        this.pageIndex--;

        if (this.pageIndex < 0) {
            this.pageIndex = 0;
        }

        this.browse(this._lastDir, this.pageIndex, function(err, res) {
            if (err) {
                XC.debugf(err, 'error');
                return;
            }

            this.updateList(res);
        }.bind(this));
    };


    /**
     * Set the layout/CSS user values.
     * @param {Object} layout
     */
    MMediaPlayer.prototype.setLayout = function(layout) {
        this._layout = {};

        if (layout && typeof layout === 'object') {
            this._layout = layout;
        }

        if (!this._layout.css || typeof this._layout.css !== 'object') {
            this._layout.css = {};
        }

        var css = this._layout.css;


        // Areas.

        if (!css.header || typeof css.header !== 'object') {
            css.header = {};
        }

        if (!css.list || typeof css.list !== 'object') {
            css.list = {};
        }

        if (!css.side || typeof css.side !== 'object') {
            css.side = {};
        }

        if (!css.cover || typeof css.cover !== 'object') {
            css.cover = {};
        }

        for (var key in css.header) {
            if (MMediaPlayer.ATTRIBUTES_ALLOWED.indexOf(key) < 0) {
                delete css.header[key];
            }
        }

        for (var key in css.list) {
            if (MMediaPlayer.ATTRIBUTES_ALLOWED.indexOf(key) < 0) {
                delete css.list[key];
            }
        }

        for (var key in css.side) {
            if (MMediaPlayer.ATTRIBUTES_ALLOWED.indexOf(key) < 0) {
                delete css.side[key];
            }
        }

        for (var key in css.cover) {
            if (MMediaPlayer.ATTRIBUTES_ALLOWED.indexOf(key) < 0) {
                delete css.cover[key];
            }
        }
    };


    /**
     * Start the update loop.
     */
    MMediaPlayer.prototype.startUpdateLoop = function() {
        if (this.running) {
            return;
        }

        this.running = true;
        this._updateLoop();
    };


    /**
     * Stop the update loop.
     */
    MMediaPlayer.prototype.stopUpdateLoop = function() {
        this.running = false;
        clearTimeout(this._timeoutUpdate);
    };


    /**
     * Hide/show the browsing container.
     * @param {Event} ev
     */
    MMediaPlayer.prototype.toggleList = function(ev) {
        if (!this.device || this.device.sys == 'bose') {
            return;
        }

        this._listContainer.toggle();

        if (this._listContainer[0].style.display == 'block') {
            if ($('#mmp-tab-browse').hasClass('active')) {
                this.browseLastOpenedDir();
            }
        }
    };


    /**
     * Update the header.
     * @param {Object} data Meta data.
     */
    MMediaPlayer.prototype.updateHeader = function(data) {
        if (!data || !this._meta) {
            return;
        }

        // Set meta info.
        if (data.info && typeof data.info === 'object') {
            var track = data.info.trackMetaData;

            if (track) {
                var src = track.albumArtURI || this.coverDefault;
                var urlNow = this._cover.css('background-image');

                if (typeof urlNow !== 'string' || urlNow.indexOf(src) < 0) {
                    this._cover.css('background-image', 'url("' + src + '")');
                }

                this._metaTitle.text(track.title || '');
                this._metaTitle.removeClass('none');

                this._metaArtist.text(track.artist || '');
            }
        }

        // Set zone info (rooms in the zone).
        if (data.zone && typeof data.zone === 'object') {
            var rooms = [];

            for (var key in data.zone) {
                rooms.push(data.zone[key]);
            }

            this._metaZone.text(rooms.join(', '));
        }
    };


    /**
     * Update the list.
     * @param {Array<Object>} data
     */
    MMediaPlayer.prototype.updateList = function(data) {
        data = data || [];

        var $back = this._listContainer.find('.back');
        $back.text('<');

        if (data.length > 0 && (data[0].parentID === '0' || !data[0].parentID)) {
            $back.text('');
        }

        var $list = this._listContainer.find('.list');
        $list.html('');

        if (this.device.sys == 'sonos' && this._lastDir && this._lastDir.length > 2 && this._lastDir.indexOf('Q:') === 0) {
            var $first = $('<li class="clear-queue"></li>');
            $first.text(XC_Text['mp_clearQueue']);
            $first.attr('id', this._lastDir);
            $first.on(XC.END_EVT, this._handleListSelect.bind(this));

            $list.append($first);
        }

        this._currentBrowsingData = data;

        var $pageNav = this._buildPageNavItem();
        $list.append($pageNav);

        // Directories to filter out.
        var dirFilter = this._getDirectoryFilter();

        for (var i = 0; i < data.length; i++) {
            var entry = data[i];
            var $item = null;

            if (entry.type == 'container') {
                if (dirFilter.indexOf(entry.id) >= 0) {
                    continue;
                }

                $item = this._buildDirectoryItem(entry);
            }
            else if (entry.type == 'item') {
                $item = this._buildFileItem(entry, i);
            }

            if ($item) {
                if (i == data.length -1) {
                    $item.addClass('last');
                }

                $item.attr('id', 'entry-' + i);
                $list.append($item);
            }
        }

        if (data.length === 0) {
            var $item = $('<li class="empty"></li>');
            $item.text(XC_Text['mp_emptyDir']);
            $list.append($item);
        }

        $pageNav = this._buildPageNavItem();
        $list.append($pageNav);

        $list.scrollTop(0);
    };


};


MMediaPlayer.ATTRIBUTES_ALLOWED = [
    'background',
    'background-color',
    'background-image',
    'border',
    'border-bottom',
    'border-left',
    'border-radius',
    'border-right',
    'border-top',
    'bottom',
    'color',
    'display',
    'font',
    'font-size',
    'font-style',
    'font-weight',
    'left',
    'margin',
    'opacity',
    'overflow',
    'padding',
    'position',
    'right',
    'text-align',
    'text-overflow',
    'top',
    'vertical-align',
    'visibility',
    'white-space'
];

MMediaPlayer.CSS = [
    '.mmp {',
        'background-color: #FFFFFF;',
        'color: #000000;',
        'font-size: 22px',
    '}',
    '.mmp .cover {',
        'background-color: #F0F0F0;',
        'background-size: contain;',
        'display: inline-block;',
    '}',
    '.mmp .meta {',
        'display: inline-block;',
        'padding: 10px;',
        'vertical-align: top;',
    '}',
    '.mmp .meta span {',
        'display: block;',
        'overflow: hidden;',
        'text-overflow: ellipsis;',
        'white-space: nowrap;',
    '}',
    '.mmp .meta .zone {',
        'color: #707070;',
        'font-size: 70%;',
        'margin-top: -5px;',
    '}',
    '.mmp .meta .title.none {',
        'font-style: italic;',
    '}',
    '.mmp .meta .artist {',
        'font-size: 70%;',
    '}',
    '.mmp .msg {',
        'background-color: #303030;',
        'color: #F0F0F0;',
        'display: none;',
        'left: 10px;',
        'padding: 10px;',
        'position: absolute;',
        'top: 10px;',
    '}',
    '.mmp .msg.info {',
        'background-color: #303030;',
    '}',
    '.mmp .msg.error {',
        'background-color: #D21C1C;',
    '}',
    '.mmp .listcontainer {',
        'background-color: #FFFFFF;',
        'display: none;',
        'left: 0;',
        'position: absolute;',
    '}',
    '.mmp .tabs {',
        'background-color: #FFFFFF;',
        'padding: 0 0 0 10px;',
    '}',
    '.mmp .tab {',
        'border-top: 10px solid #FFFFFF;',
        'display: inline-block;',
        'font-size: 80%;',
        'font-weight: bold;',
        'padding: 10px 20px 5px;',
    '}',
    '.mmp .tab.first {',
        'margin-left: 40px;',
    '}',
    '.mmp .tab.active {',
        'background-color: #F0F0F0;',
    '}',
    '.mmp .nav { position: relative; }',
    '.mmp .back {',
        'background-color: #D0D0D0;',
        'color: #F0F0F0;',
        'font-size: 200%;',
        'font-weight: bold;',
        'height: 100%;',
        'overflow: hidden;',
        'position: absolute;',
        'left: 0;',
        'text-align: center;',
        'top: 0;',
        'width: 50px;',
    '}',
    '.mmp ul {',
        'background-color: #F0F0F0;',
        'list-style: none;',
        'margin: 0 0 0 50px;',
        'overflow: auto;',
        'padding: 0;',
    '}',
    '.mmp li {',
        'border-bottom: 1px solid #D0D0D0;',
        'overflow: hidden;',
        'padding: 6px 10px;',
        'position: relative;',
        'text-overflow: ellipsis;',
        'white-space: nowrap;',
    '}',
    '.mmp li.title {',
        'font-weight: bold;',
    '}',
    '.mmp li:active {',
        'background-color: #303030;',
        'color: #F0F0F0;',
    '}',
    '.mmp li .icon {',
        'background-color: #303030;',
        'background-repeat: no-repeat;',
        'background-size: cover;',
        'display: none;',
        'height: 100%;',
        'left: 0;',
        'position: absolute;',
        'top: 0;',
        'width: 42px;',
    '}',
    '.mmp li.hasArt {',
        'padding-left: 50px;',
    '}',
    '.mmp .page-nav {',
        'border-width: 2px;',
        'font-style: italic;',
    '}',
    '.mmp .page-nav:last-child {',
        'border-top: 1px solid #D0D0D0;',
    '}',
    '.mmp .page-nav div {',
        'box-sizing: border-box;',
        'display: inline-block;',
        'width: 50%;',
    '}',
    '.mmp .page-nav .next {',
        'text-align: right;',
    '}',
    '.mmp li.playable {',
        'padding-right: 42px;',
    '}',
    '.mmp li.playable.queueable {',
        'padding-right: 84px;',
    '}',
    '.mmp li .add-to-queue {',
        'background-image: url("./images/add-to-queue.png");',
        'background-size: contain;',
        'height: 100%;',
        'position: absolute;',
        'right: 42px;',
        'top: 0;',
        'width: 42px;',
    '}',
    '.mmp li .play {',
        'background-image: url("./images/play.png");',
        'background-size: contain;',
        'height: 100%;',
        'position: absolute;',
        'right: 0;',
        'top: 0;',
        'width: 42px;',
    '}',
    '.mmp li .status {',
        'display: inline-block;',
        'margin-right: 5px;',
        'text-align: center;',
        'width: 20px;',
    '}',
    '.mmp .empty {',
        'font-style: italic;',
        'opacity: 0.7;',
    '}'
].join('');
