'use strict';


var RemoteApp = {


    KEY_EULA: 'EULA_ACCEPTED',

    LANG: {
        de: {
            confirm: 'Bestätigen',
            licenses: 'Software-Lizenzen',
            password: 'Passwort',
            passwordOnRightClick: 'Passwortschutz für Rechtsklick',
            promptTimeout: 'Sekunden, dann wird die Eingabe geschlossen',
            saveSettings: 'Übernehmen',
            settings: 'Einstellungen'
        },

        en: {
            confirm: 'Confirm',
            licenses: 'Software Licenses',
            password: 'Password',
            passwordOnRightClick: 'Password-protect right-click',
            promptTimeout: 'seconds, then the prompt will be closed',
            saveSettings: 'Apply',
            settings: 'Settings'
        }
    },

    _lang: null,
    _pwdPrompt: null,


    /**
     * Edit the startup dialog a little.
     * Since the script is linked from another
     * project, we cannot edit it directly.
     */
    _editStartupDialog: function() {
        if (RemoteController._setupNavController) {
            var $div = RemoteController._setupNavController._$div;

            // Open links in the external browser.
            var $link = $div.find('a');

            $link.click(function(ev) {
                if (ev.target.href) {
                    ev.preventDefault();
                    nw.Shell.openExternal(ev.target.href);
                }
            });

            // Remove demo button.
            var $btnDemo = $div.find('button.MNavControllerButton:contains("' + XC_Text.demo + '")');
            $btnDemo.hide();
        }
    },


    /**
     * Check if the EULA have already been accepted.
     * If not open the EULA window.
     * @return {Boolean} True if the EULA have already been accepted, false otherwise.
     */
    _eulaCheck: function() {
        var eulaAccepted = true;

        if (localStorage.getItem(this.KEY_EULA) != 'true') {
            eulaAccepted = false;

            nw.Window.open(
                '../mediola/eula.html',

                {
                    position: 'center',
                    width: 700,
                    height: 700,
                    icon: '../mediola/images/logo.png',
                    title: 'Endnutzer-Lizenzvertrag (EULA)',
                    focus: true,
                    resizable: true
                },

                function(winEULA) {
                    winEULA.on('close', function() {
                        winEULA.close(true);

                        if (localStorage.getItem(RemoteApp.KEY_EULA) == 'true') {
                            RemoteApp._start();
                        }
                        else {
                            nw.App.quit();
                        }
                    });
                }
            );
        }

        return eulaAccepted;
    },


    /**
     * Handle right-click if it is password-protected.
     * @param {Event} ev
     */
    _handleRightClick: function(ev) {
        var $dialog = $('.aio2-dialog');

        if ($dialog.length > 0 && $dialog.is(':visible')) {
            ev.preventDefault();
            ev.stopPropagation();

            return;
        }

        var self = RemoteApp;
        var usePwd = Boolean(Number(RemoteSettings.get('use-rightclick-password')));

        if (!usePwd) {
            return;
        }

        ev.preventDefault();
        ev.stopPropagation();

        if (self._pwdPrompt) {
            self._pwdPrompt.remove();
        }

        self._pwdPrompt = $('<div id="prompt-password-overlay"></div>');
        var $container = $('<div id="prompt-password"></div>');
        var $close = $('<span class="close">&times;</span>');
        var $input = $('<input type="password" />');
        var $btn = $('<button>' + self.langGet('confirm') + '</button>');

        $container.append($input);
        $container.append($btn);
        self._pwdPrompt.append($close);
        self._pwdPrompt.append($container);

        // Timeout after which to close the window.
        var timeout = Number(RemoteSettings.get('prompt-timeout')) * 1000;
        var timeoutID = 0;

        if (timeout > 0) {
            timeoutID = setTimeout(function() {
                self._pwdPrompt.hide();
            }, timeout);
        }

        $close.on('click', function(ev) {
            clearTimeout(timeoutID);
            RemoteApp._pwdPrompt.hide();
            $input.removeClass('invalid');
            $input.val('');
        });

        // Password input and validation.
        $btn.on('click', function(ev) {
            var pwdInput64 = new Buffer($input.val()).toString('base64');
            var pwd64 = RemoteSettings.get('rightclick-password');

            if (pwdInput64 == pwd64) {
                clearTimeout(timeoutID);
                RemoteController && RemoteController.showSetup();
                RemoteApp._pwdPrompt.hide();
                $input.removeClass('invalid');
                $input.val('');
            }
            else {
                $input.addClass('invalid');
                $input.focus();
            }
        });

        $input.on('keydown', function(ev) {
            if (ev.keyCode == 13) {
                $btn.click();
            }
        });

        $('body').append(self._pwdPrompt);

        $input.focus();
    },


    /**
     * Initialize the key handler.
     */
    _initKeyHandler: function() {
        $('html').keyup(function(ev) {
            switch (ev.keyCode) {
                case 27: // Escape
                    this.toggleFullscreen(false);
                    break;

                case 122: // F11
                    ev.preventDefault();
                    this.toggleFullscreen();
                    break;

                case 123: // F12
                    if (nw.App.manifest.sdk === true) {
                        ev.preventDefault();
                        nw.Window.get().showDevTools();
                    }
                    break;
            }
        }.bind(this));
    },


    /**
     * Initialize the application window menu.
     */
    _initMenu: function() {
        var win = nw.Window.get();


        // Menu items.

        var itemSetup = new nw.MenuItem({
            click: function() {
                RemoteController && RemoteController.showSetup();
            },
            label: XC_Text['setup']
        });

        var itemChangelog = new nw.MenuItem({
            click: function() {
                nw.Window.open(
                    '../mediola/changelog.html',

                    {
                        position: 'center',
                        width: 400,
                        height: 500,
                        icon: '../mediola/images/logo.png',
                        title: 'AIO REMOTE NEO (Desktop) – Changelog',
                        focus: true
                    }
                );
            },
            label: XC_Text['changelog']
        });

        var itemSettings = new nw.MenuItem({
            click: function() {
                nw.Window.open(
                    '../mediola/settings.html',

                    {
                        focus: true,
                        height: 200,
                        icon: '../mediola/images/logo.png',
                        min_height: 200,
                        min_width: 500,
                        position: 'center',
                        title: RemoteApp.langGet('settings'),
                        width: 500
                    },

                    function(winSettings) {
                        winSettings.on('closed', function() {
                            RemoteSettings.loadSettings();
                            RemoteApp._initRightClick();
                        });
                    }
                );

            },
            label: this.langGet('settings')
        });

        var itemLicenses = new nw.MenuItem({
            click: function() {
                nw.Window.open(
                    '../mediola/licenses/index.html',

                    {
                        focus: true,
                        height: 440,
                        icon: nw.App.manifest.window.icon,
                        position: 'center',
                        resizable: true,
                        title: RemoteApp.langGet('licenses'),
                        width: 800
                    }
                );
            },
            label: this.langGet('licenses')
        });

        var itemFS = new nw.MenuItem({
            click: this.toggleFullscreen.bind(this),
            label: XC_Text['fullscreen']
        });

        var itemFit = new nw.MenuItem({
            click: this.fitToPage,
            label: XC_Text['fit_to_page']
        });

        var itemStartFS = new nw.MenuItem({
            checked: Boolean(Number(localStorage.getItem('start_fullscreen'))),
            click: function() {
                var value = localStorage.getItem('start_fullscreen');
                value = Boolean(Number(value));
                localStorage.setItem('start_fullscreen', !value ? '1' : '0');

                this.checked = !value;
            },
            label: XC_Text['start_fullscreen'],
            type: 'checkbox'
        });

        var itemStartLastPos = new nw.MenuItem({
            checked: Boolean(Number(localStorage.getItem('start_lastpos'))),
            click: function() {
                var value = localStorage.getItem('start_lastpos');
                value = Boolean(Number(value));
                localStorage.setItem('start_lastpos', !value ? '1' : '0');

                this.checked = !value;
            },
            label: XC_Text['start_lastpos'],
            type: 'checkbox'
        });


        // Mac
        if (process.platform === 'darwin') {
            var nativeMenuBar = new nw.Menu({ type: 'menubar' });

            nativeMenuBar.createMacBuiltin(nw.App.manifest.appName);
            win.menu = nativeMenuBar;

            // Remove the native "About" window.
            var sm = win.menu.items[0].submenu;
            sm.removeAt(0);
            sm.insert(new nw.MenuItem({ type: 'separator' }), 0);
            sm.insert(itemLicenses, 0);
            sm.insert(itemChangelog, 0);
            sm.insert(itemSettings, 0);
            sm.insert(itemSetup, 0);

            // Expand the "Window" menu.
            var viewm = win.menu.items[2].submenu;
            viewm.insert(new nw.MenuItem({ type: 'separator' }), 0);
            viewm.insert(itemStartLastPos, 0);
            viewm.insert(itemStartFS, 0);
            viewm.insert(new nw.MenuItem({ type: 'separator' }), 0);
            viewm.insert(itemFit, 0);
            viewm.insert(itemFS, 0);
        }
        // Windows/Linux
        else {
            var menu = new nw.Menu({ type: 'menubar' });


            // Menu: AIO REMOTE NEO
            var itemMain = new nw.MenuItem({
                label: XC_Text['menu_main']
            });

            var itemQuit = new nw.MenuItem({
                click: this.quit.bind(this),
                label: XC_Text['quit']
            });

            var subMenuMain = new nw.Menu();
            subMenuMain.append(itemSetup);
            subMenuMain.append(itemSettings);
            subMenuMain.append(itemChangelog);
            subMenuMain.append(itemLicenses);
            subMenuMain.append(itemQuit);
            itemMain.submenu = subMenuMain;


            // Menu: View
            var itemView = new nw.MenuItem({
                label: XC_Text['menu_view']
            });

            var subMenuView = new nw.Menu();
            subMenuView.append(itemFS);
            subMenuView.append(itemFit);
            subMenuView.append(new nw.MenuItem({ type: 'separator' }));
            subMenuView.append(itemStartFS);
            subMenuView.append(itemStartLastPos);
            itemView.submenu = subMenuView;


            menu.append(itemMain);
            menu.append(itemView);

            this._mainMenu = menu;

            setTimeout(function() {
                win.menu = win.isFullscreen ? null : menu;
            }, 100);
        }
    },


    /**
     * Initialize right-click password-protection.
     */
    _initRightClick: function() {
        var usePwd = Boolean(Number(RemoteSettings.get('use-rightclick-password')));
        $('body').off('contextmenu', this._handleRightClick);

        if (usePwd) {
            $('body').on('contextmenu', this._handleRightClick);
        }
    },


    /**
     * Initialize the window.
     */
    _initWindow: function() {
        var win = nw.Window.get();

        win.removeAllListeners('close');
        win.removeAllListeners('resize');


        // Set the last saved width and height.

        var startFullscreen = Boolean(Number(localStorage.getItem('start_fullscreen')));
        var lastWidth = localStorage.getItem('window.width');
        var lastHeight = localStorage.getItem('window.height');

        if (!startFullscreen && lastWidth && lastHeight) {
            lastWidth = Number(lastWidth);
            lastHeight = Number(lastHeight);

            if (!isNaN(lastWidth) && !isNaN(lastHeight)) {
                win.width = lastWidth;
                win.height = lastHeight;
            }
        }
        else if (startFullscreen) {
            win.enterFullscreen();
        }


        // If option enabled: Open at last window position.

        var startLastPos = Boolean(Number(localStorage.getItem('start_lastpos')));
        var lastX = localStorage.getItem('window.x');
        var lastY = localStorage.getItem('window.y');

        if (startLastPos && lastX && lastY) {
            lastX = Number(lastX);
            lastY = Number(lastY);

            if (!isNaN(lastX) && !isNaN(lastY)) {
                win.x = lastX;
                win.y = lastY;
            }
        }


        win.on('close', function() {
            // NW.js >= v0.14.x and NW.js <= v0.12.x
            // In NW.js v0.13.x isFullscreen is a function!
            if (!win.isKioskMode && win.isFullscreen === false) {
                localStorage.setItem('window.width', win.width);
                localStorage.setItem('window.height', win.height);

                localStorage.setItem('window.x', win.x);
                localStorage.setItem('window.y', win.y);
            }

            this.quit();
        }.bind(this));
    },


    /**
     * Start the main application.
     */
    _start: function() {
        RemoteSettings.loadSettings();

        APP = new m.aioremote.App();
        XC.init();
        APP.init(true);

        var fs = require('fs');
        var dataPath = fs.dataPath().replace(/\\/g, '/');

        if (dataPath.indexOf('file:') !== 0) {
            dataPath = 'file://' + dataPath;
        }

        RemoteController.remotesPath = dataPath + '/remotes/';
        RemoteController.show();

        this._editStartupDialog();
        this._initRightClick();

        Updater.check();
    },


    /**
     * Fit the window size to the page.
     */
    fitToPage: function() {
        var page = RemoteController.currentPage;
        var w = page._contentWidth;
        var h = page._contentHeight;

        if (!w || !h || isNaN(w) || isNaN(h)) {
            return;
        }

        var win = nw.Window.get();

        win.resizeTo(
            w + (window.outerWidth - window.innerWidth),
            h + (window.outerHeight - window.innerHeight)
        );
    },


    /**
     * Initialize the application.
     */
    init: function() {
        this._initWindow();
        this.initLang();

        this.resetToIndex();
        this._initKeyHandler();
        this._initMenu();
        this.setTitle();

        if (this._eulaCheck()) {
            this._start();
        }
    },


    /**
     * Initialize the language.
     */
    initLang: function() {
        var lang = XC.getURLParam('lang');

        if (!lang && window.XCHost && window.XCHost.getLanguage) {
            lang = window.XCHost.getLanguage();
        }

        if (lang && XC_lang[lang]) {
            this._lang = lang;
            XC_Text = XC_lang[lang];
        }
        else {
            this._lang = 'de';
            XC_Text = XC_lang['de'];
        }
    },


    /**
     * Get the text for the given key.
     * @param  {String} key
     * @return {String}
     */
    langGet: function(key) {
        if (!this._lang) {
            return key;
        }

        if (!this.LANG[this._lang]) {
            return key;
        }

        return (this.LANG[this._lang][key] || key);
    },


    /**
     * Minimize the window.
     */
    minimize: function() {
        nw.Window.get().minimize();
    },


    /**
     * Quit the application.
     */
    quit: function() {
        if (XC) {
            XC.applicationWillExit(function() {
                nw.App.quit();
            });
        }
        else {
            nw.App.quit();
        }
    },


    /**
     * Reset the page to the index page.
     */
    resetToIndex: function() {
        // Only, if CURRENTPAGE has been set before.
        if (typeof localStorage.getItem('CURRENTPAGE') == 'string') {
            localStorage.setItem('CURRENTPAGE', 'index');
        }
    },


    /**
     * Set the window title.
     */
    setTitle: function() {
        var version = nw.App.manifest.version;
        $('title').text(nw.App.manifest.appName + ' – ' + version);
    },


    /**
     * Toggle fullscreen mode.
     * @param {Boolean|null} mode
     */
    toggleFullscreen: function(mode) {
        var win = nw.Window.get();

        if (typeof mode != 'undefined') {
            if (mode === true) {
                win.enterFullscreen();
            }
            else if (mode === false) {
                win.leaveFullscreen();
            }
        }
        else {
            mode = !win.isFullscreen;
            win.toggleFullscreen();
        }

        // Hide window menu in fullscreen.
        if (process.platform != 'darwin') {
            nw.Window.get().menu = mode ? null : this._mainMenu;
        }
    }


};