'use strict';


var RemoteSettings = {


    _settings: {},


    /**
     * Load the currently saved settings into the form.
     */
    _initFormValues: function() {
        for (var key in this._settings) {
            var $input = document.body.querySelector('[name="' + key + '"]');

            if ($input) {
                if ($input.type == 'checkbox') {
                    $input.checked = Boolean(Number(this._settings[key]));
                }
                else if ($input.type == 'password') {
                    $input.value = new Buffer(this._settings[key], 'base64').toString('utf8');
                }
                else {
                    $input.value = this._settings[key];
                }
            }
        }
    },


    /**
     * Initialize the UI elements.
     */
    _initUI: function() {
        this._initFormValues();

        var $btn = document.getElementById('save-settings');
        $btn.addEventListener('click', function(ev) {
            this._saveForm();
            nw.Window.get().close();
        }.bind(this));
    },


    /**
     * Save the settings.
     */
    _saveForm: function() {
        for (var key in this._settings) {
            var $input = document.body.querySelector('[name="' + key + '"]');

            if ($input) {
                if ($input.type == 'checkbox') {
                    this._settings[key] = $input.checked ? '1' : '0';
                }
                else if ($input.type == 'password') {
                    this._settings[key] = new Buffer($input.value).toString('base64');
                }
                else {
                    this._settings[key] = String($input.value);
                }

                localStorage.setItem('neo.settings.' + key, this._settings[key]);
            }
        }
    },


    /**
     * Initialize the settings UI.
     */
    init: function() {
        this.loadSettings();
        this.translate();
        this._initUI();
    },


    /**
     * Get value for key.
     * @param  {String} key
     * @return {String}
     */
    get: function(key) {
        return this._settings[key];
    },


    /**
     * Load settings.
     */
    loadSettings: function() {
        var keys = [
            'prompt-timeout',
            'rightclick-password',
            'use-rightclick-password'
        ];
        var defaultValue = [
            '60',
            '',
            '0'
        ];

        for (var i = 0; i < keys.length; i++) {
            var k = keys[i];
            this._settings[k] = localStorage.getItem('neo.settings.' + k);

            if (this._settings[k] === null) {
                this._settings[k] = defaultValue[i];
            }
        }
    },


    /**
     * Set translated texts.
     */
    translate: function() {
        var $translate = document.body.querySelectorAll('[data-text]');

        for (var i = 0; i < $translate.length; i++) {
            var $item = $translate[i];
            var key = $item.dataset.text;
            $item.textContent = RemoteApp.langGet(key);
        }
    }


};