'use strict';


var Updater = {


    _url: 'https://webservice.mediola.com/srm/getLatestRelease?name=DESKTOP_REMOTE_NEO',


    /**
     * Check for updates.
     */
    check: function() {
        if (window.XCHost && window.XCHost.getType && window.XCHost.getType() == 'node-webkit') {
            var url = Updater._url;
            var vLocal = nw.App.manifest.version;
            console.log('[Updater.check] Local version: ' + vLocal);

            $.ajax({
                async: true,
                cache: false,
                dataType: 'json',
                method: 'GET',
                url: url,

                success: function(result, textStatus, jqXHR) {
                    console.log('[Updater.check] Online version: ' + result.version);

                    var cmpResult = this.compareVersions(vLocal, result.version);

                    if (cmpResult === 1) {
                        console.debug('[Updater.check] Update available.');
                        Updater.showNotification(result);
                    }
                }.bind(this),

                error: function(jqXHR, textStatus, errorThrown) {
                    console.error('[Updater.check] ' + (errorThrown || textStatus));
                }
            });
        }
    },


    /**
     * Compare version numbers.
     * @param  {String} a Version number.
     * @param  {String} b Version number.
     * @return {Number}   -1: b is older, 0: b equals a, 1: b is newer
     */
    compareVersions: function(a, b) {
        if (!a && !b && typeof a != 'string' && typeof b != 'string') {
            return 0;
        }

        if (!a || typeof a == 'object') {
            return 1;
        }

        if (!b || typeof b == 'object') {
            return -1;
        }

        var partsA = String(a).split('-');
        var partsB = String(b).split('-');

        var suffixA = partsA[1] ? partsA[1] : false;
        var suffixB = partsB[1] ? partsB[1] : false;

        partsA = partsA[0].split('.');
        partsB = partsB[0].split('.');

        var len = Math.max(partsA.length, partsB.length);

        for (var i = 0; i < len; i++) {
            var pA = partsA[i] ? Number(partsA[i]) : 0;
            var pB = partsB[i] ? Number(partsB[i]) : 0;

            if (pB > pA) {
                return 1;
            }
            else if (pB < pA) {
                return -1;
            }
        }

        // Version numbers are the same, but the new
        // version has left beta status.
        if (suffixA == 'beta' && !suffixB) {
            return 1;
        }

        return 0;
    },


    /**
     * Show the "update available" notification.
     * @param {Object} result
     */
    showNotification: function(result) {
        var width = window.innerWidth;
        var height = window.innerHeight;

        var $div = $('<div></div>');
        $div.css({
            backgroundColor: '#ffffff',
            cursor: 'pointer',
            left: 0,
            opacity: 0.9,
            padding: '5px',
            position: 'absolute',
            textAlign: 'center',
            top: 0,
            width: '100%',
            zIndex: 100
        });

        var $span = $('<span>' + XC_Text['update_available'] + ' (' + result.version + ')</span>');
        $span.css({
            color: '#000000',
            fontSize: '14px'
        });

        $span.appendTo($div);
        $div.appendTo($('body'));

        $div.click(function() {
            nw.Shell.openExternal(result.url);
        });

        setTimeout(function() {
            $div.hide();
        }, 6000);
    }


};